; GameBoard					(c) 2015 Cygnus2
; www.cygnus2.net
;
;	**********************************************************************************************************
;	**													**
;	**				S O U R C E    C O D E    R E V I S I O N				**
;	**													**
;	**********************************************************************************************************
;	**													**
;	** 					Created by:  Vladimir RYKL					**
;	**													**
;	**********************************************************************************************************
;	**	   *		*										**
;	** Version *    Date 	*				Description					**
;	** 	   *		*										**
;	**********************************************************************************************************
;	**  V0.01  * 05/05/2015	* First release	- MCU PIC16F876	used for development purposes			**
;	** 	   *		*										**
;	** 	   *		* ROM occupation: 42.15% (3453 of 8192 words used)				**
;	**********************************************************************************************************
;	**  V0.02  * 16/06/2015	* Adapted to the use of MCU PIC16F1513, otherwise the same as V0.01		**
;	** 	   *		*										**
;	** 	   *		* ROM occupation: 84.20% (3449 of 4096 words used)				**
;	**********************************************************************************************************
;	**  V1.00  * 06/07/2015	* Change list:									**
;	** 	   *		* [1] Trimmer controlled LED brightness	implemented (F05)			**
;	** 	   *		* [2] Button controlled LED brightness removed (changed within F06 & F08)	**
;	** 	   *		* [3] Buzzer port status during sound off inverted to reduce power consumption	**
;	** 	   *		*										**
;	** 	   *		* ROM occupation: 84.08% (3444 of 4096 words used)				**
;	**********************************************************************************************************
;	**  V1.10  * 19/07/2015	* Change list:									**
;	** 	   *		* [1] Game no. 6 ("The falling stars") implemented (F14 & F15)			**
;	** 	   *		*										**
;	** 	   *		* ROM occupation: 93.68% (3837 of 4096 words used)				**
;	**********************************************************************************************************
;
	list      p=16F1513		; list directive to define processor
	#include <p16F1513.inc>		; processor specific variable definitions
;
;************************************************************************************************************************
;		M A C R O     D E F I N I T I O N
;************************************************************************************************************************
;
mREGBANK0	Macro				; Select register bank 0
		MOVLB	000H			; //
		Endm				; //
;
mREGBANK1	Macro				; Select register bank 1
		MOVLB	001H			; //
		Endm				; //
;
mREGBANK2	Macro				; Select register bank 2
		MOVLB	002H			; //
		Endm				; //
;
mREGBANK3	Macro				; Select register bank 3
		MOVLB	003H			; //
		Endm				; //
;
mREGBANK4	Macro				; Select register bank 4
		MOVLB	004H			; //
		Endm				; //
;------------------------------------------------------------------------------------------------------------------------
mPRGMEMPAGE0	Macro				; Set program memory page 0
		BCF	PCLATH,3		; //
		BCF	PCLATH,4		; //
		Endm
;
mPRGMEMPAGE1	Macro				; Set program memory page 1
		BSF	PCLATH,3		; //
		BCF	PCLATH,4		; //
		Endm
;
mPRGMEMPAGE2	Macro				; Set program memory page 2
		BCF	PCLATH,3		; //
		BSF	PCLATH,4		; //
		Endm
;
mPRGMEMPAGE3	Macro				; Set program memory page 3
		BSF	PCLATH,3		; //
		BSF	PCLATH,4		; //
		Endm
;------------------------------------------------------------------------------------------------------------------------
mCALL01		Macro	mvarADDRESS
		BSF	PCLATH,3				; Set program memory page 1
		CALL	mvarADDRESS				; Call subroutine on memory page 1
		BCF	PCLATH,3				; Set program memory page 0
		Endm
;
;************************************************************************************************************************
;		R E G I S T E R     A N D     B I T     D E F I N I T I O N
;************************************************************************************************************************
; Program control register 4
reg_ProgCtrl4		EQU	07CH
#define		bit_Function15		reg_ProgCtrl4,000H	; Function 15 requested (active)
; Program control register 3
reg_ProgCtrl3		EQU	07BH
#define		bit_Function14		reg_ProgCtrl3,007H	; Function 14 requested (active)
#define		bit_Function50		reg_ProgCtrl3,006H	; Function 50 requested (active)
#define		bit_FlashB3		reg_ProgCtrl3,005H	; Piece flashing control bits
#define		bit_FlashB2		reg_ProgCtrl3,004H	; //
#define		bit_FlashB1		reg_ProgCtrl3,003H	; //
#define		bit_FlashA3		reg_ProgCtrl3,002H	; //
#define		bit_FlashA2		reg_ProgCtrl3,001H	; //
#define		bit_FlashA1		reg_ProgCtrl3,000H	; //
; Program control register 2
reg_ProgCtrl2		EQU	07AH
#define		bit_Function13		reg_ProgCtrl2,007H	; Function 13 requested (active)
#define		bit_Function12		reg_ProgCtrl2,006H	; Function 12 requested (active)
#define		bit_Function11		reg_ProgCtrl2,005H	; Function 11 requested (active)
#define		bit_Function10		reg_ProgCtrl2,004H	; Function 10 requested (active)
#define		bit_Function09		reg_ProgCtrl2,003H	; Function 09 requested (active)
#define		bit_Function08		reg_ProgCtrl2,002H	; Function 08 requested (active)
#define		bit_Function07		reg_ProgCtrl2,001H	; Function 07 requested (active)
#define		bit_Function06		reg_ProgCtrl2,000H	; Function 06 requested (active)
; Program control register 1
reg_ProgCtrl1		EQU	079H
#define		bit_Difficulty		reg_ProgCtrl1,007H	; Game difficulty (0 - low, 1 - high)
#define		bit_Beep		reg_ProgCtrl1,006H	; Beep request
#define		bit_Freq		reg_ProgCtrl1,005H	; Tone generation auxiliary bit
con_MaskFreq		EQU	020H				; // XOR mask of bit_Freq
#define		bit_Div2		reg_ProgCtrl1,004H	; Tone generation auxiliary bit (frequency divide by two)
con_MaskDiv2		EQU	010H				; // XOR mask of bit_Div2
#define		bit_Tone2		reg_ProgCtrl1,003H	; Tone 2 flag (0 - tone off, 1 - tone on)
#define		bit_Tone1		reg_ProgCtrl1,002H	; Tone 1 flag (0 - tone off, 1 - tone on)
#define		bit_Hold		reg_ProgCtrl1,001H	; Key hold flag (0 - all keys released, 1 - any key hold)
#define		bit_Key			reg_ProgCtrl1,000H	; Key flag (0 - all keys released, 1 - any key pressed)
; Delay control register
reg_DelayCtrl		EQU	078H				; Delay expiration flags
#define		bit_LongDelay4		reg_DelayCtrl,007H	; Long delay expiration flags
#define		bit_LongDelay3		reg_DelayCtrl,006H	; //
#define		bit_LongDelay2		reg_DelayCtrl,005H	; //
#define		bit_LongDelay1		reg_DelayCtrl,004H	; //
#define		bit_ShortDelay4		reg_DelayCtrl,003H	; Short delay expiration flags
#define		bit_ShortDelay3		reg_DelayCtrl,002H	; //
#define		bit_ShortDelay2		reg_DelayCtrl,001H	; //
#define		bit_ShortDelay1		reg_DelayCtrl,000H	; //
; Shared temporary register
reg_Temp2		EQU	071H				; //
reg_Temp1		EQU	070H				; //
;------------------------------------------------------------------------------------------------------------------------
; Register bank 0
; Delay counter registers
reg0_LongDelay4H	EQU	06FH				; Long delay counter register (high & low byte)
reg0_LongDelay4L	EQU	06EH				; //
reg0_LongDelay3H	EQU	06DH				; //
reg0_LongDelay3L	EQU	06CH				; //
reg0_LongDelay2H	EQU	06BH				; //
reg0_LongDelay2L	EQU	06AH				; //
reg0_LongDelay1H	EQU	069H				; //
reg0_LongDelay1L	EQU	068H				; //
reg0_ShortDelay4	EQU	067H				; Short delay counter register
reg0_ShortDelay3	EQU	066H				; //
reg0_ShortDelay2	EQU	065H				; //
reg0_ShortDelay1	EQU	064H				; //
; Pseudo-random value generator
reg0_RND4		EQU	063H				; Pseudo-random accumulator
reg0_RND3		EQU	062H				; //
reg0_RND2		EQU	061H				; //
reg0_RND1		EQU	060H				; //
reg0_Random		EQU	05FH				; Result register
; LED driver
reg0_LEDDriver4		EQU	05EH				; LED driver register
reg0_LEDDriver3		EQU	05DH				; //
reg0_LEDDriver2		EQU	05CH				; //
reg0_LEDDriver1		EQU	05BH				; //
reg0_LEDBright		EQU	05AH				; LED brightness control register
; Key (switch) registers
reg0_KeyCode		EQU	059H				; Key code register
reg0_KeyCdStk		EQU	058H				; Key code stack
reg0_KeySamp		EQU	057H				; Key sampling register
; Temporary registers
reg0_Config		EQU	056H				; MODE selector
reg0_BeepCode		EQU	055H				; Beep / melody code
reg0_BeepPtr		EQU	054H				; Beep / melody pointer
; Game registers
reg0_PieceA3		EQU	053H				; Pieces group A
reg0_PieceA2		EQU	052H				; //
reg0_PieceA1		EQU	051H				; //
reg0_PieceB3		EQU	050H				; Pieces group B
reg0_PieceB2		EQU	04FH				; //
reg0_PieceB1		EQU	04EH				; //
reg0_PieceA3W		EQU	04DH				; Pieces group A working register
reg0_PieceA2W		EQU	04CH				; //
reg0_PieceA1W		EQU	04BH				; //
reg0_PieceB3W		EQU	04AH				; Pieces group B working register
reg0_PieceB2W		EQU	049H				; //
reg0_PieceB1W		EQU	048H				; //
reg0_CounterA		EQU	047H				; Figure A move attempt counter
reg0_CounterB		EQU	046H				; Figure B move attempt counter
reg0_Finish3		EQU	045H				; Finish figure
reg0_Finish2		EQU	044H				; //
reg0_Finish1		EQU	043H				; //
reg0_FinishCnt		EQU	042H				; Finish counter
;
reg0_Figure3		EQU	041H				; MTW measurement
reg0_Figure2		EQU	040H				; //
reg0_Figure1		EQU	03FH				; //
;
reg0_MTWA		EQU	03EH				; Move to win figure A
reg0_MTWB		EQU	03DH				; Move to win figure B
;
reg0_RankingA		EQU	03CH				; MTWA ranking
reg0_RankingB		EQU	03BH				; MTWB ranking
reg0_Ranking		EQU	03AH				; Primary MTWA ranking
reg0_RankingAlt		EQU	039H				; Alternative MTWA ranking after loop detection
reg0_MoveCode		EQU	038H				; Calculated move code of player A (computer)
reg0_MoveCodeAlt	EQU	037H				; Alternative move code in case of looping
reg0_MCBuff4		EQU	036H				; 4-stage move code buffer for loop detection
reg0_MCBuff3		EQU	035H				; //
reg0_MCBuff2		EQU	034H				; //
reg0_MCBuff1		EQU	033H				; //
;
reg0_MovePtrA		EQU	032H				; Figure A move pointer
reg0_MovePtrB		EQU	031H				; Figure B move pointer
reg0_MoveRND		EQU	030H				; Random move offset
;
reg0_LEDBrightCnt	EQU	02FH				; LED brightness working register
;
; Display RAM
reg0_DispA3		EQU	02EH				; Display RAM group A
reg0_DispA2		EQU	02DH				; //
reg0_DispA1		EQU	02CH				; //
reg0_DispB3		EQU	02BH				; Display RAM group B
reg0_DispB2		EQU	02AH				; //
reg0_DispB1		EQU	029H				; //
;
; Function phase registers
reg0_F50Phase		EQU	028H				; Function phase register
reg0_F10Phase		EQU	027H				; //
reg0_F09Phase		EQU	026H				; //
reg0_F08Phase		EQU	025H				; //
reg0_F07Phase		EQU	024H				; //
reg0_F06Phase		EQU	023H				; //
reg0_F05Phase		EQU	022H				; //
reg0_F03Phase		EQU	021H				; //
reg0_FI01Phase		EQU	020H				; //
;------------------------------------------------------------------------------------------------------------------------
; Register bank 0 - alternative register
reg0_G6Row		EQU	041H				; Game no. 6 register set
reg0_Column3		EQU	040H				; //
reg0_Column2		EQU	03FH				; //
reg0_Column1		EQU	03EH				; //
reg0_Column0		EQU	03DH				; //
;
reg0_DiceCnt		EQU	03CH				; Dice counter (pseudo-random value)
;
reg0_F15Phase		EQU	03BH				; Function phase register
reg0_F14Phase		EQU	03AH				; //
reg0_F13Phase		EQU	039H				; //
reg0_F12Phase		EQU	038H				; //
;
reg0_LEDDriver		EQU	037H				; From CreativeBoard
reg0_GameLevel		EQU	036H				; Registers shared with light effect register set
reg0_GameSpeed		EQU	035H				; //
reg0_GameStep		EQU	034H				; //
reg0_GameRnd		EQU	033H				; //
reg0_GameRepeat		EQU	032H				; //
reg0_GameBuf1		EQU	031H				; //
reg0_GameBuf2		EQU	030H				; //
;
;************************************************************************************************************************
;		P O R T     D E F I N I T I O N
;************************************************************************************************************************
;
#define		port_ROW1		PORTC,007H		; Row 1 driver
#define		port_ROW2		PORTC,006H		; Row 2 driver
#define		port_ROW3		PORTC,005H		; Row 3 driver
#define		port_ROW4		PORTC,004H		; Row 4 driver
#define		port_COL1		PORTC,003H		; Column 1 driver
#define		port_COL2		PORTC,002H		; Column 2 driver
#define		port_COL3		PORTC,001H		; Column 3 driver
#define		port_COL4		PORTC,000H		; Column 4 driver
#define		port_COL5		PORTA,006H		; Column 5 driver
#define		port_COL6		PORTA,007H		; Column 6 driver
#define		port_COL7		PORTA,005H		; Column 7 driver
#define		port_COL8		PORTA,004H		; Column 8 driver
#define		port_CFG1		PORTB,005H		; Configuration input 1
#define		port_CFG2		PORTB,006H		; Configuration input 2
#define		port_CFG3		PORTB,007H		; Configuration input 3
#define		port_BUZ		PORTB,000H		; Buzzer driver
#define		port_SW1		PORTB,003H		; Switch input 1
#define		port_SW2		PORTB,002H		; Switch input 2
#define		port_SW3		PORTB,004H		; Switch input 3
#define		port_SW4		PORTB,001H		; Switch input 4
#define		port_DIMM		PORTA,000H		; LED brightness trimmer input
#define		port_DIMM_PWR		PORTA,001H		; // power output
;
;************************************************************************************************************************
;		C O N S T A N T     D E F I N I T I O N
;************************************************************************************************************************
;
con_Game1MaxLevel	EQU	004H				; Game 1 maximum level
con_Game2MaxLevel	EQU	006H				; Game 2 maximum level
;
;************************************************************************************************************************
;		P I C 1 6 F 1 5 1 3     C O N F I G U R A T I O N     F U S E S
;************************************************************************************************************************
;
;Program Configuration Register 1
	__CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_OFF & _MCLRE_ON & _CP_OFF & _BOREN_OFF & _CLKOUTEN_OFF & _IESO_OFF & _FCMEN_OFF

;Program Configuration Register 2
	__CONFIG _CONFIG2, _WRT_OFF & _VCAPEN_OFF & _STVREN_OFF & _BORV_LO & _LPBOR_OFF & _LVP_OFF
;
;************************************************************************************************************************
;*		A P P L I C A T I O N     B E G I N									*
;************************************************************************************************************************
;
;
;		*********************************
;		*   PROGRAM  MEMORY  PAGE  0	*
;		*********************************
;
		ORG	0000H				; RESET vector
;
sa_RESET:	GOTO	sa_INIT				; Skip interrupt service routine
;
;************************************************************************************************************************
;*		I N T E R R U P T     S U B R O U T I N E S								*
;************************************************************************************************************************
;
		ORG	0004H				; Interrupt vector
; TMR0 interrupt service routine - TMR0 interrupt occurs every 256 usec
		mPRGMEMPAGE0				; --> PROGRAM MEMORY PAGE 0
		mREGBANK0				; --> REGISTER BANK 0
		BCF	INTCON,T0IF			; Clear TMR0 interrupt flag
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; Sound generator (OkHz - mute / 2kHz / 1kHz)
		BTFSC	bit_Tone1			; Tone 1 (2kHz) requested ?
		GOTO	sa_TONE10			; // YES !
		BTFSC	bit_Tone2			; Tone 2 (1kHz) requested ?
		GOTO	sa_TONE20			; // YES !
		GOTO	sa_DELAY			; // NO ! - skip tone generation
; Tone 2 (1kHz)
sa_TONE20:	MOVLW	con_MaskDiv2			; Toggle auxiliary bit_Div2 --> tone frequency / 2 (1 kHz)
		XORWF	reg_ProgCtrl1,F			; //
		BTFSS	bit_Div2			; //
		GOTO	sa_DELAY			; //
; Tone 1 (2kHz)
sa_TONE10:	MOVLW	con_MaskFreq			; Toggle bit_Freq and buzzer port
		XORWF	reg_ProgCtrl1,F			; //
		BTFSC	bit_Freq			; //
		BSF	port_BUZ			; //
		BTFSS	bit_Freq			; //
		BCF	port_BUZ			; //
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; Delay generator
; Short delay 1 (key samplig period)
sa_DELAY	DECFSZ	reg0_ShortDelay1,F		; Delay expired ?
		GOTO	sa_TMR0A			; // NO !
		BSF	bit_ShortDelay1			; // YES ! - set expiration flag
; Short delay 2
sa_TMR0A:	DECFSZ	reg0_ShortDelay2,F		; Delay expired ?
		GOTO	sa_TMR0B			; // NO !
		BSF	bit_ShortDelay2			; // YES ! - set expiration flag
; Short delay 3
sa_TMR0B:	DECFSZ	reg0_ShortDelay3,F		; Delay expired ?
		GOTO	sa_TMR0C			; // NO !
		BSF	bit_ShortDelay3			; // YES ! - set expiration flag
; Short delay 4
sa_TMR0C:	DECFSZ	reg0_ShortDelay4,F		; Delay expired ?
		GOTO	sa_TMR0D			; // NO !
		BSF	bit_ShortDelay4			; // YES ! - set expiration flag
; Long delay 1 (key hold period)
sa_TMR0D:	DECFSZ	reg0_LongDelay1L,F		; Delay expired ?
		GOTO	sa_TMR0E			; // NO !
		DECFSZ	reg0_LongDelay1H,F		; //
		GOTO	sa_TMR0E			; // NO !
		BSF	bit_LongDelay1			; // YES ! - set expiration flag
; Long delay 2 (beep duration)
sa_TMR0E:	DECFSZ	reg0_LongDelay2L,F		; Delay expired ?
		GOTO	sa_TMR0F			; // NO !
		DECFSZ	reg0_LongDelay2H,F		; //
		GOTO	sa_TMR0F			; // NO !
		BSF	bit_LongDelay2			; // YES ! - set expiration flag
; Long delay 3 (LED flashing)
sa_TMR0F:	DECFSZ	reg0_LongDelay3L,F		; Delay expired ?
		GOTO	sa_TMR0G			; // NO !
		DECFSZ	reg0_LongDelay3H,F		; //
		GOTO	sa_TMR0G			; // NO !
		BSF	bit_LongDelay3			; // YES ! - set expiration flag
; Long delay 4
sa_TMR0G:	DECFSZ	reg0_LongDelay4L,F		; Delay expired ?
		GOTO	sa_LEDMUX			; // NO !
		DECFSZ	reg0_LongDelay4H,F		; //
		GOTO	sa_LEDMUX			; // NO !
		BSF	bit_LongDelay4			; // YES ! - set expiration flag
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; LED multiplex 1:4
sa_LEDMUX:	CLRF	PCLATH				; Function phase switch
		MOVF	reg0_FI01Phase,W		; //
		ADDWF	PCL,F				; //
sa_FI01PHASE:	GOTO	sa_FI01PHASE0			; //
		GOTO	sa_FI01PHASE1			; //
		GOTO	sa_FI01PHASE2			; //
		GOTO	sa_FI01PHASE3			; //
sa_FI01PHEND:	GOTO	sa_FI01PHASE4			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_FI01PHASE) - high(sa_FI01PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
; ROW1 active
sa_FI01PHASE0:	BCF	port_ROW4			; Turn off ROW4 driver before turning on ROW1 driver
		BTFSC	reg0_LEDDriver1,007H		; reg_LEDDriver1 -> ports COL8 - COL1
		BCF	port_COL8			; Bit no. 7
		BTFSS	reg0_LEDDriver1,007H		; //
		BSF	port_COL8			; //
		BTFSC	reg0_LEDDriver1,006H		; Bit no. 6
		BCF	port_COL7			; //
		BTFSS	reg0_LEDDriver1,006H		; //
		BSF	port_COL7			; //
		BTFSC	reg0_LEDDriver1,005H		; Bit no. 5
		BCF	port_COL6			; //
		BTFSS	reg0_LEDDriver1,005H		; //
		BSF	port_COL6			; //
		BTFSC	reg0_LEDDriver1,004H		; Bit no. 4
		BCF	port_COL5			; //
		BTFSS	reg0_LEDDriver1,004H		; //
		BSF	port_COL5			; //
		BTFSC	reg0_LEDDriver1,003H		; Bit no. 3
		BCF	port_COL4			; //
		BTFSS	reg0_LEDDriver1,003H		; //
		BSF	port_COL4			; //
		BTFSC	reg0_LEDDriver1,002H		; Bit no. 2
		BCF	port_COL3			; //
		BTFSS	reg0_LEDDriver1,002H		; //
		BSF	port_COL3			; //
		BTFSC	reg0_LEDDriver1,001H		; Bit no. 1
		BCF	port_COL2			; //
		BTFSS	reg0_LEDDriver1,001H		; //
		BSF	port_COL2			; //
		BTFSC	reg0_LEDDriver1,000H		; Bit no. 0
		BCF	port_COL1			; //
		BTFSS	reg0_LEDDriver1,000H		; //
		BSF	port_COL1			; //
		BSF	port_ROW1			; Turn on ROW1 driver
		INCF	reg0_FI01Phase,F		; Phase 1
		GOTO	sa_FI01EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
; ROW2 active
sa_FI01PHASE1:	BCF	port_ROW1			; Turn off ROW1 driver before turning on ROW2 driver
		BTFSC	reg0_LEDDriver2,007H		; reg_LEDDriver1 -> ports COL8 - COL1
		BCF	port_COL8			; Bit no. 7
		BTFSS	reg0_LEDDriver2,007H		; //
		BSF	port_COL8			; //
		BTFSC	reg0_LEDDriver2,006H		; Bit no. 6
		BCF	port_COL7			; //
		BTFSS	reg0_LEDDriver2,006H		; //
		BSF	port_COL7			; //
		BTFSC	reg0_LEDDriver2,005H		; Bit no. 5
		BCF	port_COL6			; //
		BTFSS	reg0_LEDDriver2,005H		; //
		BSF	port_COL6			; //
		BTFSC	reg0_LEDDriver2,004H		; Bit no. 4
		BCF	port_COL5			; //
		BTFSS	reg0_LEDDriver2,004H		; //
		BSF	port_COL5			; //
		BTFSC	reg0_LEDDriver2,003H		; Bit no. 3
		BCF	port_COL4			; //
		BTFSS	reg0_LEDDriver2,003H		; //
		BSF	port_COL4			; //
		BTFSC	reg0_LEDDriver2,002H		; Bit no. 2
		BCF	port_COL3			; //
		BTFSS	reg0_LEDDriver2,002H		; //
		BSF	port_COL3			; //
		BTFSC	reg0_LEDDriver2,001H		; Bit no. 1
		BCF	port_COL2			; //
		BTFSS	reg0_LEDDriver2,001H		; //
		BSF	port_COL2			; //
		BTFSC	reg0_LEDDriver2,000H		; Bit no. 0
		BCF	port_COL1			; //
		BTFSS	reg0_LEDDriver2,000H		; //
		BSF	port_COL1			; //
		BSF	port_ROW2			; Turn on ROW2 driver
		INCF	reg0_FI01Phase,F		; Phase 2
		GOTO	sa_FI01EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
; ROW3 active
sa_FI01PHASE2:	BCF	port_ROW2			; Turn off ROW2 driver before turning on ROW3 driver
		BTFSC	reg0_LEDDriver3,007H		; reg_LEDDriver1 -> ports COL8 - COL1
		BCF	port_COL8			; Bit no. 7
		BTFSS	reg0_LEDDriver3,007H		; //
		BSF	port_COL8			; //
		BTFSC	reg0_LEDDriver3,006H		; Bit no. 6
		BCF	port_COL7			; //
		BTFSS	reg0_LEDDriver3,006H		; //
		BSF	port_COL7			; //
		BTFSC	reg0_LEDDriver3,005H		; Bit no. 5
		BCF	port_COL6			; //
		BTFSS	reg0_LEDDriver3,005H		; //
		BSF	port_COL6			; //
		BTFSC	reg0_LEDDriver3,004H		; Bit no. 4
		BCF	port_COL5			; //
		BTFSS	reg0_LEDDriver3,004H		; //
		BSF	port_COL5			; //
		BTFSC	reg0_LEDDriver3,003H		; Bit no. 3
		BCF	port_COL4			; //
		BTFSS	reg0_LEDDriver3,003H		; //
		BSF	port_COL4			; //
		BTFSC	reg0_LEDDriver3,002H		; Bit no. 2
		BCF	port_COL3			; //
		BTFSS	reg0_LEDDriver3,002H		; //
		BSF	port_COL3			; //
		BTFSC	reg0_LEDDriver3,001H		; Bit no. 1
		BCF	port_COL2			; //
		BTFSS	reg0_LEDDriver3,001H		; //
		BSF	port_COL2			; //
		BTFSC	reg0_LEDDriver3,000H		; Bit no. 0
		BCF	port_COL1			; //
		BTFSS	reg0_LEDDriver3,000H		; //
		BSF	port_COL1			; //
		BSF	port_ROW3			; Turn on ROW3 driver
		INCF	reg0_FI01Phase,F		; Phase 3
		GOTO	sa_FI01EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
; ROW4 active
sa_FI01PHASE3:	BCF	port_ROW3			; Turn off ROW3 driver before turning on ROW4 driver
		BTFSC	reg0_LEDDriver4,007H		; reg_LEDDriver1 -> ports COL8 - COL1
		BCF	port_COL8			; Bit no. 7
		BTFSS	reg0_LEDDriver4,007H		; //
		BSF	port_COL8			; //
		BTFSC	reg0_LEDDriver4,006H		; Bit no. 6
		BCF	port_COL7			; //
		BTFSS	reg0_LEDDriver4,006H		; //
		BSF	port_COL7			; //
		BTFSC	reg0_LEDDriver4,005H		; Bit no. 5
		BCF	port_COL6			; //
		BTFSS	reg0_LEDDriver4,005H		; //
		BSF	port_COL6			; //
		BTFSC	reg0_LEDDriver4,004H		; Bit no. 4
		BCF	port_COL5			; //
		BTFSS	reg0_LEDDriver4,004H		; //
		BSF	port_COL5			; //
		BTFSC	reg0_LEDDriver4,003H		; Bit no. 3
		BCF	port_COL4			; //
		BTFSS	reg0_LEDDriver4,003H		; //
		BSF	port_COL4			; //
		BTFSC	reg0_LEDDriver4,002H		; Bit no. 2
		BCF	port_COL3			; //
		BTFSS	reg0_LEDDriver4,002H		; //
		BSF	port_COL3			; //
		BTFSC	reg0_LEDDriver4,001H		; Bit no. 1
		BCF	port_COL2			; //
		BTFSS	reg0_LEDDriver4,001H		; //
		BSF	port_COL2			; //
		BTFSC	reg0_LEDDriver4,000H		; Bit no. 0
		BCF	port_COL1			; //
		BTFSS	reg0_LEDDriver4,000H		; //
		BSF	port_COL1			; //
		BSF	port_ROW4			; Turn on ROW4 driver
		MOVF	reg0_LEDBright,W		; Maximum LED brightness ?
		MOVLW	000H				; // YES ! - phase 0
		BTFSS	STATUS,Z			; //
		MOVLW	004H				; // NO ! - phase 4
		MOVWF	reg0_FI01Phase			; //
		GOTO	sa_FI01EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
; LED brightness control
sa_FI01PHASE4:	BCF	port_ROW4			; Turn off ROW4 driver
		INCF	reg0_LEDBrightCnt,W		; Brightness counter overflow ?
		MOVWF	reg0_LEDBrightCnt		; //
		SUBWF	reg0_LEDBright,W		; //
		BTFSC	STATUS,C			; //
		GOTO	sa_FI01EXIT			; // NO ! - exit function
		CLRF	reg0_LEDBrightCnt		; // YES ! - reinitialize brightness counter
		CLRF	reg0_FI01Phase			; Phase 0
		GOTO	sa_FI01EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_FI01EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; Interrupt leaving - restore W, STATUS and PCLATH automatically from shadow register bank31
		RETFIE	
;
;************************************************************************************************************************
;*		S Y S T E M     I N I T I A L I Z A T I O N								*
;************************************************************************************************************************
;
;		---------------------
;		HARDWARE MODULE SETUP
;		---------------------
;
; I/O port setup
sa_INIT:	MOVLW	0FFH				; Initialize port drivers
		MOVWF	PORTA				; // DIMM_PWR ON
		MOVLW	0FFH				; //
		MOVWF	PORTB				; //
		MOVLW	00FH				; //
		MOVWF	PORTC				; //
		mREGBANK3				; --> REGISTER BANK 1
		MOVLW	001H				; RA0 = analog input
		MOVWF	ANSELA				; //
		CLRF	ANSELB				; //
		CLRF	ANSELC				; //
		mREGBANK4				; --> REGISTER BANK 4
		MOVLW	0E0H				; Weak pull-up on RB7, RB6, RB5
		MOVWF	WPUB				; //
		mREGBANK1				; --> REGISTER BANK 1
		MOVLW	001H				; RA0 = input
		MOVWF	TRISA				; //
		MOVLW	0FEH				; RB0 = output
		MOVWF	TRISB				; //
		MOVLW	000H				; RCx = output
		MOVWF	TRISC				; //
; Oscillator and TMR0 setup
		MOVLW	050H				; Clock source Fosc/4 ~ 2 MHz, prescaler 1:2; weak pull-up enabled
		MOVWF	OPTION_REG			; //
		MOVLW	072H				; Internal Fosc = 8 MHz
		MOVWF	OSCCON				; //
		mREGBANK0				; --> REGISTER BANK 0
		CLRF	TMR0				; Clear TMR0
;
;		---------------------------
;		MCU REGISTER INITIALIZATION
;		---------------------------
;
		CLRF	reg_ProgCtrl1			; Clear program control registers
		CLRF	reg_ProgCtrl2			; //
		CLRF	reg_ProgCtrl3			; //
		CLRF	reg_ProgCtrl4			; //
		MOVLW	0FFH				; Set all delay expiration flags
		MOVWF	reg_DelayCtrl			; //
		MOVLW	0FFH				; Initialize MODE selector with invalid value to ensure
		MOVWF	reg0_Config			; // appropriate configuration after RESET(fuction 05)
		CLRF	reg0_KeyCode			; Clear key code register
		CLRF	reg0_KeyCdStk			; Clear key code stack
		MOVLW	055H				; Initialize key sampling register
		MOVWF	reg0_KeySamp			; //
		CLRF	reg0_FI01Phase			; Clear all function phase registers
		CLRF	reg0_F03Phase			; //
		CLRF	reg0_F05Phase			; //
		CLRF	reg0_F06Phase			; //
		CLRF	reg0_F07Phase			; //
		CLRF	reg0_F08Phase			; //
		CLRF	reg0_F09Phase			; //
		CLRF	reg0_F10Phase			; //
		CLRF	reg0_LEDDriver4			; Initialize LED driver (all LEDs turned off)
		CLRF	reg0_LEDDriver3			; //
		CLRF	reg0_LEDDriver2			; //
		CLRF	reg0_LEDDriver1			; //
		CLRF	reg0_LEDBright			; Maximum LED brightness
		CLRF	reg0_RND4			; Clear pseudo-random generator
		CLRF	reg0_RND3			; //
		CLRF	reg0_RND2			; //
		CLRF	reg0_RND1			; //
		BSF	INTCON,T0IE			; TMR0 interrupt enable
		BSF	INTCON,GIE			; Global interrupt enable
;
;************************************************************************************************************************
;*		M A I N     P R O G R A M     L O O P									*
;************************************************************************************************************************
;
sa_MAINLOOP
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 01: pseudo-random generator (values 0x00 - 0x0F)
;
		MOVLW	014H				; Generate pseudo-random number
		ADDWF	reg0_RND1,F			; //
		MOVLW	082H				; //
		ADDWF	reg0_RND2,F			; //
		MOVLW	03CH				; //
		ADDWF	reg0_RND3,F			; //
		MOVLW	001H				; //
		ADDWF	reg0_RND4,F			; //
		MOVF	reg0_RND1,W			; //
		XORWF	reg0_RND2,W			; //
		XORWF	reg0_RND3,W			; //
		XORWF	reg0_RND4,W			; //
		ANDLW	0F0H				; //
		BTFSC	bit_Function09			; If function 09 or 10 active, random value range 0x00 - 0x03
		ANDLW	030H				; //
		BTFSC	bit_Function10			; //
		ANDLW	030H				; //
		MOVWF	reg0_Random			; Store pseudo-random value
		SWAPF	reg0_Random,F			; //
;------------------------------------------------------------------------------------------------------------------------
sa_F01EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 02: Key (switch) sampling - 8 subsequent samples have to be equal to recognize valid key code
; Code 0x00 - switch released / code 0xXX - switch pressed
; Key sampling period about 5 msec, key response 8 x 5 = 40 msec
;
;		| 7. bit | 6. bit | 5. bit | 4. bit | 3. bit | 2. bit | 1. bit | 0. bit |
; reg0_KeyCode	|   x    |   x    |   x    |  S102  |  S100  |  S101  |  S103  |   x    |
;
		BTFSS	bit_ShortDelay1			; Key sampling period expired ?
		GOTO	sa_F02EXIT			; // NO ! - exit function
		MOVLW	013H				; // YES ! - reinitialize delay for 5 msec
		MOVWF	reg0_ShortDelay1		; //
		BCF	bit_ShortDelay1			; //
;
		COMF	PORTB,W				; Store current key status into temporary register
		ANDLW	01EH				; // (mask key bits)
		MOVWF	reg_Temp1			; //
;
		XORWF	reg0_KeyCdStk,W			; Compare two consecutive key code samples
		BCF	STATUS,C			; If two consecutive samples equal, shift in '0', otherwise	
		BTFSS	STATUS,Z			; // shift in '1'
		BSF	STATUS,C			; //
		RLF	reg0_KeySamp,F			; //
;
		MOVF	reg_Temp1,W			; Restore current key status from temporary register
		MOVF	reg0_KeySamp,F			; Last 8 samples equal ?
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_KeyCode			; // YES ! - update key code with new value
		MOVWF	reg0_KeyCdStk			; //
;
		MOVLW	000H				; reg_KeyCode = 0x00 --> bit_Key = 0 & bit_Hold = 0
		XORWF	reg0_KeyCode,W			; reg_KeyCode <> 0x00 --> bit_Key = 1
		BTFSS	STATUS,Z			; //
		GOTO	sa_F0201			; //
		BCF	bit_Key				; bit_Key = 0
		BCF	bit_Hold			; bit_Hold = 0
		MOVLW	010H				; Initialize hold timer for 1 sec
		MOVWF	reg0_LongDelay1H		; //
		MOVLW	042H				; //
		MOVWF	reg0_LongDelay1L		; //
		BCF	bit_LongDelay1			; Clear expiration flag
		GOTO	sa_F02EXIT			; //
sa_F0201:	BSF	bit_Key				; bit_Key = 1
		BTFSC	bit_LongDelay1			; Hold delay expired ?
		BSF	bit_Hold			; // YES ! - bit_Hold = 1
;------------------------------------------------------------------------------------------------------------------------
sa_F02EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 03: Sound (beep & melody) generator - maximum beep duration about 4 sec
;	|-----------|-----------|------------------|
;	| bit_Tone2 | bit_Tone1 | output frequency |
;	|-----------|-----------|------------------|
;	|     0     |     0     |       0 kHz      |
;	|     0     |     1     |       2 kHz      |
;	|     1     |     0     |       1 kHz      |
;	|     1     |     1     |    not allowed   |
;	|-----------|-----------|------------------|
;
		BTFSS	bit_Beep			; Beep requested ?
		GOTO	sa_F03EXIT			; // NO ! - skip function
		MOVLW	high(sa_F03PHASE)		; // YES ! - function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg0_F03Phase,W			; //
		ADDWF	PCL,F				; //
sa_F03PHASE:	GOTO	sa_F03PHASE0			; //
		GOTO	sa_F03PHASE1			; //
sa_F03PHEND:	GOTO	sa_F03PHASE2			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F03PHASE) - high(sa_F03PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F03PHASE0:	CALL	sa_F03TABLE1			; Initialize beep / melody pointer
		MOVWF	reg0_BeepPtr			; //
		INCF	reg0_F03Phase,F			; Phase 1
		GOTO	sa_F03EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F03PHASE1:	CALL	sa_F03TABLE2			; Get first / next beep code from table
		MOVWF	reg_Temp1			; //
		INCF	reg0_BeepPtr,F			; //
		XORLW	0FFH				; Delimiter found ?
		BTFSC	STATUS,Z			; //
		GOTO	sa_F03PH100			; // YES ! - stop sound
		MOVF	reg_Temp1,W			; // NO ! - continue melody
		ANDLW	03FH				; Initialize beep duration
		MOVWF	reg0_LongDelay2H		; //
		MOVLW	07FH				; //
		MOVWF	reg0_LongDelay2L		; //
		BCF	bit_LongDelay2			; //
		RLF	reg_Temp1,F			; Tone frequency decoding
		BTFSS	STATUS,C			; //
		BCF	bit_Tone2			; //
		BTFSC	STATUS,C			; //
		BSF	bit_Tone2			; // 1 kHz
		RLF	reg_Temp1,F			; //
		BTFSS	STATUS,C			; //
		BCF	bit_Tone1			; //
		BTFSC	STATUS,C			; //
		BSF	bit_Tone1			; // 2 kHz
		INCF	reg0_F03Phase,F			; Phase 1
		GOTO	sa_F03EXIT			; Exit function
; Stop sound generator
sa_F03PH100:	BSF	port_BUZ			; Set buzzer driver high to reduce current consumption
		BCF	bit_Tone1			; Stop tone generation
		BCF	bit_Tone2			; //
		BCF	bit_Beep			; Stop function 04
		CLRF	reg0_F03Phase			; Phase 0
		GOTO	sa_F03EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F03PHASE2:	BTFSC	bit_LongDelay2			; Beep duration expired ?
		DECF	reg0_F03Phase,F			; // YES ! - phase 1
		GOTO	sa_F03EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F03TABLE1:	MOVLW	high(sa_F03TABBEG1)		; Beep / melody decoder
		MOVWF	PCLATH				; //
		MOVF	reg0_BeepCode,W			; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F03TABBEG1) - high(sa_F03TABEND1) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F03TABBEG1:	RETLW	low(sa_BEEP00-sa_BEEP00)	; Beep / melody code 0x00
		RETLW	low(sa_BEEP01-sa_BEEP00)	; Beep / melody code 0x01
		RETLW	low(sa_BEEP02-sa_BEEP00)	; Beep / melody code 0x02
		RETLW	low(sa_BEEP03-sa_BEEP00)	; Beep / melody code 0x03
		RETLW	low(sa_BEEP04-sa_BEEP00)	; Beep / melody code 0x04
		RETLW	low(sa_BEEP05-sa_BEEP00)	; Beep / melody code 0x05
		RETLW	low(sa_BEEP06-sa_BEEP00)	; Beep / melody code 0x06
		RETLW	low(sa_BEEP07-sa_BEEP00)	; Beep / melody code 0x07
		RETLW	low(sa_BEEP08-sa_BEEP00)	; Beep / melody code 0x08
		RETLW	low(sa_BEEP09-sa_BEEP00)	; Beep / melody code 0x09
		RETLW	low(sa_BEEP10-sa_BEEP00)	; Beep / melody code 0x0A
sa_F03TABEND1:	RETLW	low(sa_BEEP11-sa_BEEP00)	; Beep / melody code 0x0B
;------------------------------------------------------------------------------------------------------------------------
; Beep encoding
;		|  7. bit   |  6. bit   |  5. bit   |  4. bit   |  3. bit   |  2. bit   |  1. bit   |  0. bit   |
; reg0_BeepCode	| bit_Tone2 | bit_Tone1 | - - - - - - - - - - - - - - beep duration - - - - - - - - - - - - - - |
;
sa_F03TABLE2:	MOVLW	high(sa_F03TABBEG2)		; Beep code table
		MOVWF	PCLATH				; //
		MOVF	reg0_BeepPtr,W			; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F03TABBEG2) - high(sa_F03TABEND2) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F03TABBEG2:
sa_BEEP00:	RETLW	007H				; Beep / melody code 0x00 - 0 kHz
		RETLW	0FFH				; // delimiter
sa_BEEP01:	RETLW	042H				; Beep / melody code 0x01 - 2 kHz / 0.1 sec
		RETLW	0FFH				; // delimiter
sa_BEEP02:	RETLW	044H				; Beep / melody code 0x02 - 2 kHz / 0.2 sec
		RETLW	0FFH				; // delimiter
sa_BEEP03:	RETLW	048H				; Beep / melody code 0x03 - 2 kHz / 0.5 sec
		RETLW	0FFH				; // delimiter
sa_BEEP04:	RETLW	050H				; Beep / melody code 0x04 - 2 kHz / 1 sec
		RETLW	0FFH				; // delimiter
sa_BEEP05:	RETLW	082H				; Beep / melody code 0x05 - 1 kHz / 0.1 sec
		RETLW	0FFH				; // delimiter
sa_BEEP06:	RETLW	084H				; Beep / melody code 0x06 - 1 kHz / 0.2 sec
		RETLW	0FFH				; // delimiter
sa_BEEP07:	RETLW	088H				; Beep / melody code 0x07 - 1 kHz / 0.5 sec
		RETLW	0FFH				; // delimiter
sa_BEEP08:	RETLW	090H				; Beep / melody code 0x08 - 1 kHz / 1 sec
		RETLW	0FFH				; // delimiter
sa_BEEP09:	RETLW	041H				; Beep / melody code 0x09 - 2 kHz / 15 msec
		RETLW	0FFH				; // delimiter
sa_BEEP10:	RETLW	082H				; Beep / melody code 0x0A - game begin melody
		RETLW	050H				; //
		RETLW	0FFH				; // delimiter
sa_BEEP11:	RETLW	041H				; Beep / melody code 0x0B - game over melody
		RETLW	081H				; //
		RETLW	001H				; //
		RETLW	041H				; //
		RETLW	081H				; //
		RETLW	001H				; //
		RETLW	041H				; //
		RETLW	081H				; //
		RETLW	001H				; //
		RETLW	041H				; //
		RETLW	081H				; //
		RETLW	001H				; //
		RETLW	041H				; //
		RETLW	081H				; //
		RETLW	001H				; //
		RETLW	041H				; //
		RETLW	081H				; //
		RETLW	001H				; //
		RETLW	041H				; //
		RETLW	081H				; //
		RETLW	001H				; //
sa_F03TABEND2:	RETLW	0FFH				; // delimiter
;------------------------------------------------------------------------------------------------------------------------
sa_F03EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 04: MODE selector
;		| 7. bit | 6. bit | 5. bit | 4. bit | 3. bit | 2. bit | 1. bit | 0. bit |
; reg0_Config	|   x    |   x    |   x    |   x    |   x    |  CFG3  |  CFG2  |  CFG1  |
;
		SWAPF	PORTB,W				; MODE changed ?
		MOVWF	reg_Temp1			; // (align CFGx right)
		RRF	reg_Temp1,W			; //
		ANDLW	007H				; // ignore bit 7 - 3
		MOVWF	reg_Temp1			; //
		XORWF	reg0_Config,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F04EXIT			; // NO ! - exit function
;
		CLRF	reg_ProgCtrl2			; // YES ! - change working mode
		CLRF	reg_ProgCtrl3			; //
		CLRF	reg_ProgCtrl4			; //
		MOVF	reg_Temp1,W			; Update MODE selector
		MOVWF	reg0_Config			; //
		MOVLW	high(sa_F04CONFIG)		; Configuration switch
		MOVWF	PCLATH				; //
		MOVF	reg0_Config,W			; //
		ADDWF	PCL,F				; //
sa_F04CONFIG:	GOTO	sa_F04CFG0			; //
		GOTO	sa_F04CFG1			; //
		GOTO	sa_F04CFG2			; //
		GOTO	sa_F04CFG3			; //
		GOTO	sa_F04CFG4			; //
		GOTO	sa_F04CFG5			; //
		GOTO	sa_F04CFG6			; //
sa_F04CFGEND:	GOTO	sa_F04CFG7			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F04CONFIG) - high(sa_F04CFGEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
; Configuration 000 - Game no. 1 - player A (computer) vs. player B, difficulty high
sa_F04CFG0:	BSF	bit_Function08
		CLRF	reg0_F08Phase
		BSF	bit_Difficulty
		BSF	bit_Function07
		CLRF	reg0_F07Phase
		GOTO	sa_F04GRID
;
; Configuration 001 - Game no. 2 - player A (computer) vs. player B, difficulty low
sa_F04CFG1:	BSF	bit_Function08
		CLRF	reg0_F08Phase
		BCF	bit_Difficulty
		BSF	bit_Function07
		CLRF	reg0_F07Phase
		GOTO	sa_F04GRID
;
; Configuration 010 - Game no. 3 - player A vs. player B
sa_F04CFG2:	BSF	bit_Function06
		CLRF	reg0_F06Phase
		BSF	bit_Function07
		CLRF	reg0_F07Phase
		GOTO	sa_F04GRID
;
; Configuration 011 - Game no. 4 - Are you quick enough ?
sa_F04CFG3:	BSF	bit_Function09
		CLRF	reg0_F09Phase
		BSF	bit_Function11
		GOTO	sa_F04EXIT			; Exit function
;
; Configuration 100 - Game no. 5 - Memory game
sa_F04CFG4:	BSF	bit_Function10
		CLRF	reg0_F10Phase
		BSF	bit_Function11
		GOTO	sa_F04EXIT			; Exit function
;
; Configuration 101 - Game no. 6 - The falling stars
sa_F04CFG5:	BSF	bit_Function14
		CLRF	reg0_F14Phase
		GOTO	sa_F04EXIT			; Exit function
;
; Configuration 110 - Electronic dice A
sa_F04CFG6:	BSF	bit_Function12
		CLRF	reg0_F12Phase
		GOTO	sa_F04DICE
;
; Configuration 111 - Electronic dice B
sa_F04CFG7:	BSF	bit_Function13
		CLRF	reg0_F13Phase
		GOTO	sa_F04DICE
;------------------------------------------------------------------------------------------------------------------------
sa_F04GRID:	MOVLW	000H				; Grid initial figure (initial figure A)
		MOVWF	reg0_DispA1			; //
		MOVLW	032H				; //
		MOVWF	reg0_DispA2			; //
		MOVLW	023H				; //
		MOVWF	reg0_DispA3			; //
		MOVLW	033H				; Initial figure B
		MOVWF	reg0_DispB1			; //
		MOVLW	001H				; //
		MOVWF	reg0_DispB2			; //
		MOVLW	010H				; //
		MOVWF	reg0_DispB3			; //
		MOVLW	0C0H				; No LED flashing
		ANDWF	reg_ProgCtrl3,F			; //
		GOTO	sa_F04EXIT			; Exit function
;
sa_F04DICE:	MOVLW	020H				; Dice initial figure (number 5)
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	08FH				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	055H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	088H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F04EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F04EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 05: LED brightness control by trimmer (A/D conversion)
		MOVLW	high(sa_F05PHASE)		; Function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg0_F05Phase,W			; //
		ADDWF	PCL,F				; //
sa_F05PHASE:	GOTO	sa_F05PHASE0			; //
sa_F05PHEND:	GOTO	sa_F05PHASE1			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F05PHASE) - high(sa_F05PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F05PHASE0:	mREGBANK1				; --> REGISTER BANK 1
		MOVLW	001H				; A/D converter ON, A/D channel 1
		MOVWF	ADCON0				; //
		MOVLW	060H				; Conversion result left justified, conversion clock Fosc/64
		MOVLW	ADCON1				; //
		BSF	ADCON0,GO			; Start A/D conversion
		mREGBANK0				; --> REGISTER BANK 0
		INCF	reg0_F05Phase,F			; Phase 1
		GOTO	sa_F05EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F05PHASE1:	mREGBANK1				; --> REGISTER BANK 1
		BTFSS	ADCON0,GO			; A/D conversion finnished ?
		GOTO	sa_F05PH0100			; // YES !
		mREGBANK0				; --> REGISTER BANK 0
		GOTO	sa_F05EXIT			; Exit function
;
sa_F05PH0100:	MOVF	ADRES0H,W			; Save A/D conversion result (high byte only)
		mREGBANK0				; --> REGISTER BANK 0
		MOVWF	reg_Temp1			; //
		MOVF	reg_Temp1,W			; If ADC value < 0x20 -> LED brightness value = 0x3F
		SUBLW	040H				; //
		MOVLW	03FH				; //
		MOVWF	reg0_LEDBright			; //
		BTFSC	STATUS,C			; //
		GOTO	sa_F05PH0199			; //
		MOVF	reg_Temp1,W			; If ADC value < 0x50 -> LED brightness value = 0x1F
		SUBLW	060H				; //
		MOVLW	01FH				; //
		MOVWF	reg0_LEDBright			; //
		BTFSC	STATUS,C			; //
		GOTO	sa_F05PH0199			; //
		MOVF	reg_Temp1,W			; If ADC value < 0x80 -> LED brightness value = 0x0F
		SUBLW	080H				; //
		MOVLW	00FH				; //
		MOVWF	reg0_LEDBright			; //
		BTFSC	STATUS,C			; //
		GOTO	sa_F05PH0199			; //
		MOVF	reg_Temp1,W			; If ADC value < 0xB0 -> LED brightness value = 0x05
		SUBLW	0A0H				; //
		MOVLW	005H				; //
		MOVWF	reg0_LEDBright			; //
		BTFSC	STATUS,C			; //
		GOTO	sa_F05PH0199			; //
		MOVF	reg_Temp1,W			; If ADC value < 0xE0 -> LED brightness value = 0x02
		SUBLW	0C0H				; //
		MOVLW	002H				; //
		MOVWF	reg0_LEDBright			; //
		BTFSC	STATUS,C			; //
		GOTO	sa_F05PH0199			; //
		MOVLW	000H				; If ADC value > 0xE0 -> LED brightness value = 0x00
		MOVWF	reg0_LEDBright			; //
sa_F05PH0199:	CLRF	reg0_F05Phase			; Phase 0
		GOTO	sa_F05EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F05EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 06: Game "GRID player A vs. player B"
;
		BTFSS	bit_Function06			; Function 06 requested ?
		GOTO	sa_F06EXIT			; // NO ! - skip function
;
		MOVLW	high(sa_F06PHASE)		; Function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg0_F06Phase,W			; //
		ADDWF	PCL,F				; //
sa_F06PHASE:	GOTO	sa_F06PHASE00			; //
		GOTO	sa_F06PHASE01			; //
		GOTO	sa_F06PHASE02			; //
		GOTO	sa_F06PHASE03			; //
		GOTO	sa_F06PHASE04			; //
		GOTO	sa_F06PHASE05			; //
		GOTO	sa_F06PHASE06			; //
		GOTO	sa_F06PHASE07			; //
		GOTO	sa_F06PHASE08			; //
		GOTO	sa_F06PHASE09			; //
		GOTO	sa_F06PHASE10			; //
sa_F06PHEND:	GOTO	sa_F06PHASE11			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F06PHASE) - high(sa_F06PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F06PHASE00:	BTFSS	bit_Key				; Any key pressed ?
		GOTO	sa_F06EXIT			; // NO ! - exit function
		MOVLW	00AH				; // YES ! - play the game - game begin melody
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
;
		MOVLW	0FFH				; Initialize 4-stage move code buffer
		MOVWF	reg0_MCBuff4			; //
		MOVWF	reg0_MCBuff3			; //
		MOVWF	reg0_MCBuff2			; //
		MOVWF	reg0_MCBuff1			; //
;
		MOVLW	000H				; Initial figure A
		MOVWF	reg0_PieceA1			; //
		MOVWF	reg0_DispA1			; //
		MOVWF	reg0_PieceA1W			; //
		MOVLW	032H				; //
		MOVWF	reg0_PieceA2			; //
		MOVWF	reg0_DispA2			; //
		MOVWF	reg0_PieceA2W			; //
		MOVLW	023H				; //
		MOVWF	reg0_PieceA3			; //
		MOVWF	reg0_DispA3			; //
		MOVWF	reg0_PieceA3W			; //
		MOVLW	033H				; Initial figure B
		MOVWF	reg0_PieceB1			; //
		MOVWF	reg0_DispB1			; //
		MOVWF	reg0_PieceB1W			; //
		MOVLW	001H				; //
		MOVWF	reg0_PieceB2			; //
		MOVWF	reg0_DispB2			; //
		MOVWF	reg0_PieceB2W			; //
		MOVLW	010H				; //
		MOVWF	reg0_PieceB3			; //
		MOVWF	reg0_DispB3			; //
		MOVWF	reg0_PieceB3W			; //
;
		MOVLW	0C0H				; No LED flashing
		ANDWF	reg_ProgCtrl3,F			; //
		INCF	reg0_F06Phase,F			; Phase 1
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F06PHASE01:	BTFSS	bit_Key				; All keys released ?
		INCF	reg0_F06Phase,F			; // YES ! - phase 2
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F06PHASE02:	BTFSS	bit_Key				; Key pressed ?
		GOTO	sa_F06EXIT			; // NO ! - exit function
		MOVLW	002H				; S103 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH0200			; // YES ! - player A begins
		MOVLW	008H				; S100 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH0210			; // YES ! - player B begins
		MOVLW	007H				; Beep Not OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		DECF	reg0_F06Phase,F			; Phase 1
		GOTO	sa_F06EXIT			; Exit function
;
sa_F06PH0200:	MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		BSF	bit_FlashA1			; Player A begins
		CLRF	reg0_CounterA			; Initialize piece Ax pointer
		CLRF	reg0_MovePtrA			; Initialize move A pointer
		MOVLW	003H				; Phase 3
		MOVWF	reg0_F06Phase			; //
		GOTO	sa_F06EXIT			; Exit function
;
sa_F06PH0210:	MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		BSF	bit_FlashB1			; Player B begins
		CLRF	reg0_CounterB			; Initialize piece Bx pointer
		CLRF	reg0_MovePtrB			; Initialize move B pointer
		MOVLW	007H				; Phase 7
		MOVWF	reg0_F06Phase			; //
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
; 		* PLAYER A *
sa_F06PHASE03:	BTFSS	bit_Key				; All keys released ?
		INCF	reg0_F06Phase,F			; // YES ! - phase 4
		BTFSC	bit_Hold			; Any key hold ?
		CLRF	reg0_F06Phase			; // YES ! - phase 0 / restart game
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F06PHASE04:	BTFSS	bit_Key				; Key pressed ?
		GOTO	sa_F06EXIT			; // NO ! - exit function
		MOVLW	002H				; S103 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH0400			; // YES ! - select piece Ax to be moved
		MOVLW	010H				; S102 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH0410			; // YES ! - move selected piece Ax
		MOVLW	007H				; Beep Not OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		DECF	reg0_F06Phase,F			; Phase 3
		GOTO	sa_F06EXIT			; Exit function
;
sa_F06PH0400:	MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
;
sa_F06PH0401:	INCF	reg0_CounterA,F			; Increment piece Ax pointer
		MOVLW	003H				; // value range 0x00 - 0x02
		XORWF	reg0_CounterA,W			; //
		BTFSC	STATUS,Z			; //
		CLRF	reg0_CounterA			; //
		CLRF	reg0_MovePtrA			; Initialize move A pointer
; Check if selected piece Ax is movable
		CALL	sub_AMOVABLE			; Piece Ax movable ?
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH0401			; // NO ! - find next movable piece A
;
		CLRF	reg0_MovePtrA			; Initialize move A pointer
		MOVLW	0C0H				; Clear all LED flashing bits
		ANDWF	reg_ProgCtrl3,F			; //
		MOVLW	000H				; Ax pointer = 0x00 -> A1 flashing
		XORWF	reg0_CounterA,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashA1			; //
		MOVLW	001H				; Ax pointer = 0x01 -> A2 flashing
		XORWF	reg0_CounterA,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashA2			; //
		MOVLW	002H				; Ax pointer = 0x02 -> A3 flashing
		XORWF	reg0_CounterA,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashA3			; //
		DECF	reg0_F06Phase,F			; Phase 3
		GOTO	sa_F06EXIT			; Exit function
;
sa_F06PH0410:	MOVLW	006H				; Phase 6
		MOVWF	reg0_F06Phase			; //
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F06PHASE05:	BTFSS	bit_Key				; All keys released ?
		INCF	reg0_F06Phase,F			; // YES ! - phase 6
		BTFSC	bit_Hold			; Any key hold ?
		CLRF	reg0_F06Phase			; // YES ! - phase 0 / restart game
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F06PHASE06:	BTFSS	bit_Key				; Key pressed ?
		GOTO	sa_F06EXIT			; // NO ! - exit function
		MOVLW	002H				; S103 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH0600			; // YES ! - confirm move
		MOVLW	010H				; S102 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH0650			; // YES ! - move selected piece Ax
		MOVLW	007H				; Beep Not OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		DECF	reg0_F06Phase,F			; Phase 5
		GOTO	sa_F06EXIT			; Exit function
;
sa_F06PH0600:	MOVF	reg0_PieceA1W,W			; Update figure A
		MOVWF	reg0_PieceA1			; //
		MOVF	reg0_PieceA2W,W			; //
		MOVWF	reg0_PieceA2			; //
		MOVF	reg0_PieceA3W,W			; //
		MOVWF	reg0_PieceA3			; //
; Game over detection
		CALL	sub_AGOVER			; Game over ?
		BTFSS	STATUS,Z			; //
		GOTO	sa_F06PH0620			; // NO !
;
		MOVLW	00BH				; Game over melody
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	0C0H				; Game over --> figure A flashing
		ANDWF	reg_ProgCtrl3,F			; //
		BSF	bit_FlashA1			; //
		BSF	bit_FlashA2			; //
		BSF	bit_FlashA3			; //
		MOVLW	00BH				; Phase 11
		MOVWF	reg0_F06Phase			; //
		GOTO	sa_F06EXIT			; Exit function
; Switch to player B
sa_F06PH0620:	MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	0FFH				; Initialize piece Bx pointer
		MOVWF	reg0_CounterB			; //
sa_F06PH0621:	INCF	reg0_CounterB,F			; Increment piece Bx pointer
		MOVLW	003H				; // value range 0x00 - 0x02
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		CLRF	reg0_CounterB			; //
		CLRF	reg0_MovePtrB			; Initialize move B pointer
; Check if selected piece Bx is movable
		CALL	sub_BMOVABLE			; Piece Bx movable ?
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH0621			; // NO ! - find next movable piece B
		CLRF	reg0_MovePtrB			; Initialize move B pointer
		MOVLW	0C0H				; Clear all LED flashing bits
		ANDWF	reg_ProgCtrl3,F			; //
		MOVLW	000H				; Bx pointer = 0x00 -> B1 flashing
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashB1			; //
		MOVLW	001H				; Bx pointer = 0x01 -> B2 flashing
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashB2			; //
		MOVLW	002H				; Bx pointer = 0x02 -> B3 flashing
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashB3			; //
		MOVLW	007H				; Phase 7
		MOVWF	reg0_F06Phase			; //
		GOTO	sa_F06EXIT			; Exit function
;
sa_F06PH0650:	MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	000H				; Ax pointer = 0x00 -> offset = 0x08
		XORWF	reg0_CounterA,W			; //
		MOVLW	008H				; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH0655			; //
		MOVLW	001H				; Ax pointer = 0x01 -> offset = 0x04
		XORWF	reg0_CounterA,W			; //
		MOVLW	004H				; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH0655			; //
		MOVLW	000H				; Ax pointer = 0x02 -> offset = 0x00
;
sa_F06PH0655:	ADDWF	reg0_MovePtrA,W			; Piece A1 table pointer
		MOVWF	reg_Temp1			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece A1 from table
		ADDWF	reg0_PieceA1,W			; Calculate new position of piece A1
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceA1W			; //
		MOVLW	00CH				; Piece A2 table offset
		ADDWF	reg_Temp1,F			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece A2 from table
		ADDWF	reg0_PieceA2,W			; Calculate new position of piece A2
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceA2W			; //
		MOVLW	00CH				; Piece A3 table offset
		ADDWF	reg_Temp1,F			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece A3 from table
		ADDWF	reg0_PieceA3,W			; Calculate new position of piece A3
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceA3W			; //
		INCF	reg0_MovePtrA,F			; Increment move A pointer
		MOVLW	004H				; // value range 0x00 - 0x03
		XORWF	reg0_MovePtrA,W			; //
		BTFSC	STATUS,Z			; //
		CLRF	reg0_MovePtrA			; //
		MOVF	reg0_PieceB1,W			; Important before collision check !!!
		MOVWF	reg0_PieceB1W			; // Move figure B into working register
		MOVF	reg0_PieceB2,W			; //
		MOVWF	reg0_PieceB2W			; //
		MOVF	reg0_PieceB3,W			; //
		MOVWF	reg0_PieceB3W			; //
		mCALL01	sub_Collision			; Collision check
		BTFSC	STATUS,Z			; Collision detected ?
		GOTO	sa_F06PH0650			; // YES ! - search for valid move
		MOVF	reg0_PieceA1W,W			; // NO ! - update display
		MOVWF	reg0_DispA1			; //
		MOVF	reg0_PieceA2W,W			; //
		MOVWF	reg0_DispA2			; //
		MOVF	reg0_PieceA3W,W			; //
		MOVWF	reg0_DispA3			; //
		DECF	reg0_F06Phase,F			; Phase 5
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
; 		* PLAYER B *
sa_F06PHASE07:	BTFSS	bit_Key				; All keys released ?
		INCF	reg0_F06Phase,F			; // YES ! - phase 4
		BTFSC	bit_Hold			; Any key hold ?
		CLRF	reg0_F06Phase			; // YES ! - phase 0 / restart game
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F06PHASE08:	BTFSS	bit_Key				; Key pressed ?
		GOTO	sa_F06EXIT			; // NO ! - exit function
		MOVLW	008H				; S100 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH0800			; // YES ! - select piece Bx to be moved
		MOVLW	004H				; S101 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH0810			; // YES ! - move selected piece Bx
		MOVLW	007H				; Beep Not OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		DECF	reg0_F06Phase,F			; Phase 7
		GOTO	sa_F06EXIT			; Exit function
;
sa_F06PH0800:	MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
;
sa_F06PH0801:	INCF	reg0_CounterB,F			; Increment piece Bx pointer
		MOVLW	003H				; // value range 0x00 - 0x02
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		CLRF	reg0_CounterB			; //
		CLRF	reg0_MovePtrB			; Initialize move B pointer
; Check if selected piece Bx is movable
		CALL	sub_BMOVABLE			; Piece Bx movable ?
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH0801			; // NO ! - find next movable piece B
;
		CLRF	reg0_MovePtrB			; Initialize move B pointer
		MOVLW	0C0H				; Clear all LED flashing bits
		ANDWF	reg_ProgCtrl3,F			; //
		MOVLW	000H				; Bx pointer = 0x00 -> B1 flashing
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashB1			; //
		MOVLW	001H				; Bx pointer = 0x01 -> B2 flashing
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashB2			; //
		MOVLW	002H				; Bx pointer = 0x02 -> B3 flashing
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashB3			; //
		DECF	reg0_F06Phase,F			; Phase 7
		GOTO	sa_F06EXIT			; Exit function
;
sa_F06PH0810:	MOVLW	00AH				; Phase 10
		MOVWF	reg0_F06Phase			; //
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F06PHASE09:	BTFSS	bit_Key				; All keys released ?
		INCF	reg0_F06Phase,F			; // YES ! - phase 10
		BTFSC	bit_Hold			; Any key hold ?
		CLRF	reg0_F06Phase			; // YES ! - phase 0 / restart game
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F06PHASE10:	BTFSS	bit_Key				; Key pressed ?
		GOTO	sa_F06EXIT			; // NO ! - exit function
		MOVLW	008H				; S100 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH1000			; // YES ! - confirm move
		MOVLW	004H				; S101 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH1050			; // YES ! - move selected piece Bx
		MOVLW	007H				; Beep Not OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		DECF	reg0_F06Phase,F			; Phase 5
		GOTO	sa_F06EXIT			; Exit function
;
sa_F06PH1000:	MOVF	reg0_PieceB1W,W			; Update figure B
		MOVWF	reg0_PieceB1			; //
		MOVF	reg0_PieceB2W,W			; //
		MOVWF	reg0_PieceB2			; //
		MOVF	reg0_PieceB3W,W			; //
		MOVWF	reg0_PieceB3			; //
; Game over detection
		CALL	sub_BGOVER			; Game over ?
		BTFSS	STATUS,Z			; //
		GOTO	sa_F06PH1020			; // NO !
;
		MOVLW	00BH				; Game over melody
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	0C0H				; Game over --> figure B flashing
		ANDWF	reg_ProgCtrl3,F			; //
		BSF	bit_FlashB1			; //
		BSF	bit_FlashB2			; //
		BSF	bit_FlashB3			; //
		MOVLW	00BH				; Phase 11
		MOVWF	reg0_F06Phase			; //
		GOTO	sa_F06EXIT			; Exit function
; Switch to player A
sa_F06PH1020:	MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	0FFH				; Initialize piece Ax pointer
		MOVWF	reg0_CounterA			; //
sa_F06PH1021:	INCF	reg0_CounterA,F			; Increment piece Ax pointer
		MOVLW	003H				; // value range 0x00 - 0x02
		XORWF	reg0_CounterA,W			; //
		BTFSC	STATUS,Z			; //
		CLRF	reg0_CounterA			; //
		CLRF	reg0_MovePtrA			; Initialize move A pointer
; Check if selected piece Ax is movable
		CALL	sub_AMOVABLE			; Piece Ax movable ?
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH1021			; // NO ! - find next movable piece A
		CLRF	reg0_MovePtrA			; Initialize move A pointer
		MOVLW	0C0H				; Clear all LED flashing bits
		ANDWF	reg_ProgCtrl3,F			; //
		MOVLW	000H				; Ax pointer = 0x00 -> A1 flashing
		XORWF	reg0_CounterA,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashA1			; //
		MOVLW	001H				; Ax pointer = 0x01 -> A2 flashing
		XORWF	reg0_CounterA,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashA2			; //
		MOVLW	002H				; Ax pointer = 0x02 -> A3 flashing
		XORWF	reg0_CounterA,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashA3			; //
		MOVLW	003H				; Phase 3
		MOVWF	reg0_F06Phase			; //
		GOTO	sa_F06EXIT			; Exit function
;
sa_F06PH1050:	MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	000H				; Bx pointer = 0x00 -> offset = 0x08
		XORWF	reg0_CounterB,W			; //
		MOVLW	008H				; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH1055			; //
		MOVLW	001H				; Bx pointer = 0x01 -> offset = 0x04
		XORWF	reg0_CounterB,W			; //
		MOVLW	004H				; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F06PH1055			; //
		MOVLW	000H				; Bx pointer = 0x02 -> offset = 0x00
;
sa_F06PH1055:	ADDWF	reg0_MovePtrB,W			; Piece B1 table pointer
		MOVWF	reg_Temp1			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece B1 from table
		ADDWF	reg0_PieceB1,W			; Calculate new position of piece B1
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceB1W			; //
		MOVLW	00CH				; Piece B2 table offset
		ADDWF	reg_Temp1,F			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece B2 from table
		ADDWF	reg0_PieceB2,W			; Calculate new position of piece B2
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceB2W			; //
		MOVLW	00CH				; Piece B3 table offset
		ADDWF	reg_Temp1,F			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece B3 from table
		ADDWF	reg0_PieceB3,W			; Calculate new position of piece B3
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceB3W			; //
		INCF	reg0_MovePtrB,F			; Increment move B pointer
		MOVLW	004H				; // value range 0x00 - 0x03
		XORWF	reg0_MovePtrB,W			; //
		BTFSC	STATUS,Z			; //
		CLRF	reg0_MovePtrB			; //
		MOVF	reg0_PieceA1,W			; Important before collision check !!!
		MOVWF	reg0_PieceA1W			; // Move figure A into working register
		MOVF	reg0_PieceA2,W			; //
		MOVWF	reg0_PieceA2W			; //
		MOVF	reg0_PieceA3,W			; //
		MOVWF	reg0_PieceA3W			; //
		mCALL01	sub_Collision			; Collision check
		BTFSC	STATUS,Z			; Collision detected ?
		GOTO	sa_F06PH1050			; // YES ! - search for valid move
		MOVF	reg0_PieceB1W,W			; // NO ! - update display
		MOVWF	reg0_DispB1			; //
		MOVF	reg0_PieceB2W,W			; //
		MOVWF	reg0_DispB2			; //
		MOVF	reg0_PieceB3W,W			; //
		MOVWF	reg0_DispB3			; //
		DECF	reg0_F06Phase,F			; Phase 9
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F06PHASE11:	BTFSS	bit_Key				; All keys released ?
		CLRF	reg0_F06Phase			; // YES ! - phase 0
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F06EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 07: LED controller @ game no. 1 / 2 / 3
;
		BTFSS	bit_Function07			; Function 07 requested ?
		GOTO	sa_F07EXIT			; // NO ! - skip function
;
		MOVLW	high(sa_F07PHASE)		; Function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg0_F07Phase,W			; //
		ADDWF	PCL,F				; //
sa_F07PHASE:	GOTO	sa_F07PHASE0			; //
sa_F07PHEND:	GOTO	sa_F07PHASE1			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F07PHASE) - high(sa_F07PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F07PHASE0:	BTFSS	bit_LongDelay3			; Delay expired ?
		GOTO	sa_F07EXIT			; // NO ! - exit function
		MOVLW	002H				; // YES ! - initialize LED flashing timer
		MOVWF	reg0_LongDelay3H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay3L		; //
		BCF	bit_LongDelay3			; //
;
		MOVF	reg0_DispA1,W			; Display figure A - ignore flashing bits
		CALL	sub_DISPLAY			; Piece A1
		MOVF	reg_Temp2,W			; //
		MOVWF	reg0_LEDDriver1			; //
		MOVF	reg_Temp1,W			; //
		MOVWF	reg0_LEDDriver3			; //
		MOVF	reg0_DispA2,W			; Piece A2
		CALL	sub_DISPLAY			; //
		MOVF	reg_Temp2,W			; //
		IORWF	reg0_LEDDriver1,F		; //
		MOVF	reg_Temp1,W			; //
		IORWF	reg0_LEDDriver3,F		; //
		MOVF	reg0_DispA3,W			; Piece A3
		CALL	sub_DISPLAY			; //
		MOVF	reg_Temp2,W			; //
		IORWF	reg0_LEDDriver1,F		; //
		MOVF	reg_Temp1,W			; //
		IORWF	reg0_LEDDriver3,F		; //
;
		MOVF	reg0_DispB1,W			; Display figure B - ignore flashing bits
		CALL	sub_DISPLAY			; Piece B1
		MOVF	reg_Temp2,W			; //
		MOVWF	reg0_LEDDriver2			; //
		MOVF	reg_Temp1,W			; //
		MOVWF	reg0_LEDDriver4			; //
		MOVF	reg0_DispB2,W			; Piece B2
		CALL	sub_DISPLAY			; //
		MOVF	reg_Temp2,W			; //
		IORWF	reg0_LEDDriver2,F		; //
		MOVF	reg_Temp1,W			; //
		IORWF	reg0_LEDDriver4,F		; //
		MOVF	reg0_DispB3,W			; Piece B3
		CALL	sub_DISPLAY			; //
		MOVF	reg_Temp2,W			; //
		IORWF	reg0_LEDDriver2,F		; //
		MOVF	reg_Temp1,W			; //
		IORWF	reg0_LEDDriver4,F		; //
;
		INCF	reg0_F07Phase,F			; Phase 1
		GOTO	sa_F07EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F07PHASE1:	BTFSS	bit_LongDelay3			; Delay expired ?
		GOTO	sa_F07EXIT			; // NO ! - exit function
		MOVLW	002H				; // YES ! - initialize LED flashing timer
		MOVWF	reg0_LongDelay3H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay3L		; //
		BCF	bit_LongDelay3			; //
;
		CLRF	reg0_LEDDriver1			; Display figure A
		CLRF	reg0_LEDDriver3			; //
		MOVF	reg0_DispA1,W			; Piece A1
		CALL	sub_DISPLAY			; //
		MOVF	reg_Temp2,W			; //
		BTFSS	bit_FlashA1			; //
		IORWF	reg0_LEDDriver1,F		; //
		MOVF	reg_Temp1,W			; //
		BTFSS	bit_FlashA1			; //
		IORWF	reg0_LEDDriver3,F		; //
		MOVF	reg0_DispA2,W			; Piece A2
		CALL	sub_DISPLAY			; //
		MOVF	reg_Temp2,W			; //
		BTFSS	bit_FlashA2			; //
		IORWF	reg0_LEDDriver1,F		; //
		MOVF	reg_Temp1,W			; //
		BTFSS	bit_FlashA2			; //
		IORWF	reg0_LEDDriver3,F		; //
		MOVF	reg0_DispA3,W			; Piece A3
		CALL	sub_DISPLAY			; //
		MOVF	reg_Temp2,W			; //
		BTFSS	bit_FlashA3			; //
		IORWF	reg0_LEDDriver1,F		; //
		MOVF	reg_Temp1,W			; //
		BTFSS	bit_FlashA3			; //
		IORWF	reg0_LEDDriver3,F		; //
;
		CLRF	reg0_LEDDriver2			; Display figure B
		CLRF	reg0_LEDDriver4			; //
		MOVF	reg0_DispB1,W			; Piece B1
		CALL	sub_DISPLAY			; //
		MOVF	reg_Temp2,W			; //
		BTFSS	bit_FlashB1			; //
		IORWF	reg0_LEDDriver2,F		; //
		MOVF	reg_Temp1,W			; //
		BTFSS	bit_FlashB1			; //
		IORWF	reg0_LEDDriver4,F		; //
		MOVF	reg0_DispB2,W			; Piece B2
		CALL	sub_DISPLAY			; //
		MOVF	reg_Temp2,W			; //
		BTFSS	bit_FlashB2			; //
		IORWF	reg0_LEDDriver2,F		; //
		MOVF	reg_Temp1,W			; //
		BTFSS	bit_FlashB2			; //
		IORWF	reg0_LEDDriver4,F		; //
		MOVF	reg0_DispB3,W			; Piece B3
		CALL	sub_DISPLAY			; //
		MOVF	reg_Temp2,W			; //
		BTFSS	bit_FlashB3			; //
		IORWF	reg0_LEDDriver2,F		; //
		MOVF	reg_Temp1,W			; //
		BTFSS	bit_FlashB3			; //
		IORWF	reg0_LEDDriver4,F		; //
;
		CLRF	reg0_F07Phase			; Phase 0
		GOTO	sa_F07EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sub_DISPLAY:	MOVWF	reg_Temp1			; Single piece decoder converting YX-coordinates into
		MOVWF	reg_Temp2			; // reg0_LEDDriverX value
		RRF	reg_Temp2,F			; //
		RRF	reg_Temp2,W			; //
		IORWF	reg_Temp1,F			; //
		RLF	reg_Temp1,W			; //
		ANDLW	01EH				; //
		MOVWF	reg_Temp1			; //
		CALL	sa_F07TAB1			; //
		MOVWF	reg_Temp2			; //
		INCF	reg_Temp1,F			; //
		CALL	sa_F07TAB1			; //
		MOVWF	reg_Temp1			; //
		RETURN					; //
;
sa_F07TAB1:	MOVLW	high(sa_F07TAB1BEG)		; Decoder
		MOVWF	PCLATH				; //
		MOVF	reg_Temp1,W			; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F07TAB1BEG) - high(sa_F07TAB1END) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F07TAB1BEG:	RETLW	000H				; 00 00
		RETLW	080H				; //
		RETLW	080H				; 00 01
		RETLW	000H				; //
		RETLW	000H				; 00 02
		RETLW	008H				; //
		RETLW	008H				; 00 03
		RETLW	000H				; //
		RETLW	000H				; 01 00
		RETLW	040H				; //
		RETLW	040H				; 01 01
		RETLW	000H				; //
		RETLW	000H				; 01 02
		RETLW	004H				; //
		RETLW	004H				; 01 03
		RETLW	000H				; //
		RETLW	000H				; 02 00
		RETLW	020H				; //
		RETLW	020H				; 02 01
		RETLW	000H				; //
		RETLW	000H				; 02 02
		RETLW	002H				; //
		RETLW	002H				; 02 03
		RETLW	000H				; //
		RETLW	000H				; 03 00
		RETLW	010H				; //
		RETLW	010H				; 03 01
		RETLW	000H				; //
		RETLW	000H				; 03 02
		RETLW	001H				; //
		RETLW	001H				; 03 03
sa_F07TAB1END:	RETLW	000H				; //
;------------------------------------------------------------------------------------------------------------------------
sa_F07EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 08: Game "GRID player vs. computer"
;
		BTFSS	bit_Function08			; Function 08 requested ?
		GOTO	sa_F08EXIT			; // NO ! - skip function
;
		MOVLW	high(sa_F08PHASE)		; Function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg0_F08Phase,W			; //
		ADDWF	PCL,F				; //
sa_F08PHASE:	GOTO	sa_F08PHASE00			; //
		GOTO	sa_F08PHASE01			; //
		GOTO	sa_F08PHASE02			; //
		GOTO	sa_F08PHASE03			; //
		GOTO	sa_F08PHASE04			; //
		GOTO	sa_F08PHASE05			; //
		GOTO	sa_F08PHASE06			; //
		GOTO	sa_F08PHASE07			; //
		GOTO	sa_F08PHASE08			; //
		GOTO	sa_F08PHASE09			; //
		GOTO	sa_F08PHASE10			; //
sa_F08PHEND:	GOTO	sa_F08PHASE11			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F08PHASE) - high(sa_F08PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F08PHASE00:	BTFSS	bit_Key				; Any key pressed ?
		GOTO	sa_F08EXIT			; // NO ! - exit function
		MOVLW	00AH				; // YES ! - play the game - game begin melody
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
;
		MOVLW	0FFH				; Initialize 4-stage move code buffer
		MOVWF	reg0_MCBuff4			; //
		MOVWF	reg0_MCBuff3			; //
		MOVWF	reg0_MCBuff2			; //
		MOVWF	reg0_MCBuff1			; //
;
		MOVLW	000H				; Initial figure B
		MOVWF	reg0_PieceB1			; //
		MOVWF	reg0_DispB1			; //
		MOVWF	reg0_PieceB1W			; //
		MOVLW	032H				; //
		MOVWF	reg0_PieceB2			; //
		MOVWF	reg0_DispB2			; //
		MOVWF	reg0_PieceB2W			; //
		MOVLW	023H				; //
		MOVWF	reg0_PieceB3			; //
		MOVWF	reg0_DispB3			; //
		MOVWF	reg0_PieceB3W			; //
		MOVLW	033H				; Initial figure A
		MOVWF	reg0_PieceA1			; //
		MOVWF	reg0_DispA1			; //
		MOVWF	reg0_PieceA1W			; //
		MOVLW	001H				; //
		MOVWF	reg0_PieceA2			; //
		MOVWF	reg0_DispA2			; //
		MOVWF	reg0_PieceA2W			; //
		MOVLW	010H				; //
		MOVWF	reg0_PieceA3			; //
		MOVWF	reg0_DispA3			; //
		MOVWF	reg0_PieceA3W			; //
;
		MOVLW	0C0H				; No LED flashing
		ANDWF	reg_ProgCtrl3,F			; //
		INCF	reg0_F08Phase,F			; Phase 1
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE01:	BTFSS	bit_Key				; All keys released ?
		INCF	reg0_F08Phase,F			; // YES ! - phase 2
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE02:	BTFSS	bit_Key				; Key pressed ?
		GOTO	sa_F08EXIT			; // NO ! - exit function
		MOVLW	008H				; S100 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F08PH0200			; // YES ! - player A begins
		MOVLW	002H				; S103 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F08PH0210			; // YES ! - player B begins
		MOVLW	007H				; Beep Not OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		DECF	reg0_F08Phase,F			; Phase 1
		GOTO	sa_F08EXIT			; Exit function
;
sa_F08PH0200:	MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	003H				; Phase 3
		MOVWF	reg0_F08Phase			; //
		GOTO	sa_F08EXIT			; Exit function
;
sa_F08PH0210:	MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		BSF	bit_FlashB1			; Player B begins
		CLRF	reg0_CounterB			; Initialize piece Bx pointer
		CLRF	reg0_MovePtrB			; Initialize move B pointer
		MOVLW	007H				; Phase 7
		MOVWF	reg0_F08Phase			; //
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
; 		* PLAYER A *
sa_F08PHASE03:	BTFSS	bit_Key				; All keys released ?
		GOTO	sa_F08PH0300			; // YES ! - phase 4
		BTFSC	bit_Hold			; Any key hold ?
		GOTO	sa_F08PH0310			; // YES ! - phase 0 / restart game
		GOTO	sa_F08EXIT			; // NO ! - exit function
;
sa_F08PH0300:	BSF	bit_Function50			; Run move A calculation
		CLRF	reg0_F50Phase			; // IMPORTANT 
		INCF	reg0_F08Phase,F			; Phase 4
		GOTO	sa_F08EXIT			; Exit function
;
sa_F08PH0310:	CLRF	reg0_F08Phase			; Phase 0
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE04:	BTFSC	bit_Function50			; Move calculation finished ?
		GOTO	sa_F08EXIT			; // NO ! - exit function
;
		DECF	reg0_MoveCode,W			; Piece Ax flashing
		MOVWF	reg_Temp1			; //
		RRF	reg_Temp1,F			; //
		RRF	reg_Temp1,W			; //
		ANDLW	003H				; //
		MOVWF	reg_Temp1			; //
		MOVLW	002H				; Move code = 12 - 9 ?
		XORWF	reg_Temp1,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashA1			; // YES ! - piece A1 flashing
		MOVLW	001H				; Move code = 8 - 5 ?
		XORWF	reg_Temp1,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashA2			; // YES ! - piece A2 flashing
		MOVLW	000H				; Move code = 4 - 1 ?
		XORWF	reg_Temp1,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashA3			; // YES ! - piece A3 flashing
;
		MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
;
		MOVLW	010H				; Initialize timer for 1 sec
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	042H				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; Clear expiration flag
;
		INCF	reg0_F08Phase,F			; Phase 5
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE05:	BTFSS	bit_LongDelay4			; Delay expired ?
		GOTO	sa_F08EXIT			; // NO ! - exit function
; MOVE PLAYER A
		DECF	reg0_MoveCode,W			; Update figure A
		MOVWF	reg_Temp1			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece A1 from table
		ADDWF	reg0_PieceA1,W			; Calculate new position of piece A1
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceA1			; //
		MOVWF	reg0_PieceA1W			; //
		MOVWF	reg0_DispA1			; //
		MOVLW	00CH				; Piece A2 table offset
		ADDWF	reg_Temp1,F			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece A2 from table
		ADDWF	reg0_PieceA2,W			; Calculate new position of piece A2
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceA2			; //
		MOVWF	reg0_PieceA2W			; //
		MOVWF	reg0_DispA2			; //
		MOVLW	00CH				; Piece A3 table offset
		ADDWF	reg_Temp1,F			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece A3 from table
		ADDWF	reg0_PieceA3,W			; Calculate new position of piece A3
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceA3			; //
		MOVWF	reg0_PieceA3W			; //
		MOVWF	reg0_DispA3			; //
; Game over detection
		CALL	sub_AGOVER			; Game over ?
		BTFSS	STATUS,Z			; //
		GOTO	sa_F08PH0520			; // NO !
;
		MOVLW	00BH				; Game over melody
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	0C0H				; Game over --> figure A flashing
		ANDWF	reg_ProgCtrl3,F			; //
		BSF	bit_FlashA1			; //
		BSF	bit_FlashA2			; //
		BSF	bit_FlashA3			; //
		MOVLW	00BH				; Phase 11
		MOVWF	reg0_F08Phase			; //
		GOTO	sa_F08EXIT			; Exit function
;
sa_F08PH0520:	MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
;
		MOVLW	010H				; Initialize timer for 1 sec
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	042H				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; Clear expiration flag
;
		INCF	reg0_F08Phase,F			; Phase 6
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE06:	BTFSS	bit_LongDelay4			; Delay expired ?
		GOTO	sa_F08EXIT			; // NO ! - exit function
; Switch to player B
		MOVLW	0FFH				; Initialize piece Bx pointer
		MOVWF	reg0_CounterB			; //
sa_F08PH0600:	INCF	reg0_CounterB,F			; Increment piece Bx pointer
		MOVLW	003H				; // value range 0x00 - 0x02
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		CLRF	reg0_CounterB			; //
		CLRF	reg0_MovePtrB			; Initialize move B pointer
; Check if selected piece Bx is movable
		CALL	sub_BMOVABLE			; Piece Bx movable ?
		BTFSC	STATUS,Z			; //
		GOTO	sa_F08PH0600			; // NO ! - find next movable piece B
		CLRF	reg0_MovePtrB			; Initialize move B pointer
		MOVLW	0C0H				; Clear all LED flashing bits
		ANDWF	reg_ProgCtrl3,F			; //
		MOVLW	000H				; Bx pointer = 0x00 -> B1 flashing
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashB1			; //
		MOVLW	001H				; Bx pointer = 0x01 -> B2 flashing
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashB2			; //
		MOVLW	002H				; Bx pointer = 0x02 -> B3 flashing
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashB3			; //
		MOVLW	007H				; Phase 7
		MOVWF	reg0_F08Phase			; //
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
; 		* PLAYER B *
sa_F08PHASE07:	BTFSS	bit_Key				; All keys released ?
		INCF	reg0_F08Phase,F			; // YES ! - phase 4
		BTFSC	bit_Hold			; Any key hold ?
		CLRF	reg0_F08Phase			; // YES ! - phase 0 / restart game
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE08:	BTFSS	bit_Key				; Key pressed ?
		GOTO	sa_F08EXIT			; // NO ! - exit function
		MOVLW	002H				; S103 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F08PH0800			; // YES ! - select piece Bx to be moved
		MOVLW	010H				; S102 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F08PH0810			; // YES ! - move selected piece Bx
		MOVLW	007H				; Beep Not OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		DECF	reg0_F08Phase,F			; Phase 7
		GOTO	sa_F08EXIT			; Exit function
;
sa_F08PH0800:	MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
;
sa_F08PH0801:	INCF	reg0_CounterB,F			; Increment piece Bx pointer
		MOVLW	003H				; // value range 0x00 - 0x02
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		CLRF	reg0_CounterB			; //
		CLRF	reg0_MovePtrB			; Initialize move B pointer
; Check if selected piece Bx is movable
		CALL	sub_BMOVABLE			; Piece Bx movable ?
		BTFSC	STATUS,Z			; //
		GOTO	sa_F08PH0801			; // NO ! - find next movable piece B
;
		CLRF	reg0_MovePtrB			; Initialize move B pointer
		MOVLW	0C0H				; Clear all LED flashing bits
		ANDWF	reg_ProgCtrl3,F			; //
		MOVLW	000H				; Bx pointer = 0x00 -> B1 flashing
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashB1			; //
		MOVLW	001H				; Bx pointer = 0x01 -> B2 flashing
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashB2			; //
		MOVLW	002H				; Bx pointer = 0x02 -> B3 flashing
		XORWF	reg0_CounterB,W			; //
		BTFSC	STATUS,Z			; //
		BSF	bit_FlashB3			; //
		DECF	reg0_F08Phase,F			; Phase 7
		GOTO	sa_F08EXIT			; Exit function
;
sa_F08PH0810:	MOVLW	00AH				; Phase 10
		MOVWF	reg0_F08Phase			; //
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE09:	BTFSS	bit_Key				; All keys released ?
		INCF	reg0_F08Phase,F			; // YES ! - phase 10
		BTFSC	bit_Hold			; Any key hold ?
		CLRF	reg0_F08Phase			; // YES ! - phase 0 / restart game
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE10:	BTFSS	bit_Key				; Key pressed ?
		GOTO	sa_F08EXIT			; // NO ! - exit function
		MOVLW	002H				; S103 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F08PH1000			; // YES ! - confirm move
		MOVLW	010H				; S102 pressed ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F08PH1050			; // YES ! - move selected piece Bx
		MOVLW	007H				; Beep Not OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		DECF	reg0_F08Phase,F			; Phase 5
		GOTO	sa_F08EXIT			; Exit function
;
sa_F08PH1000:	MOVF	reg0_PieceB1W,W			; Update figure B
		MOVWF	reg0_PieceB1			; //
		MOVF	reg0_PieceB2W,W			; //
		MOVWF	reg0_PieceB2			; //
		MOVF	reg0_PieceB3W,W			; //
		MOVWF	reg0_PieceB3			; //
; Game over detection
		CALL	sub_BGOVER			; Game over ?
		BTFSS	STATUS,Z			; //
		GOTO	sa_F08PH1020			; // NO !
;
		MOVLW	00BH				; Game over melody
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	0C0H				; Game over --> figure B flashing
		ANDWF	reg_ProgCtrl3,F			; //
		BSF	bit_FlashB1			; //
		BSF	bit_FlashB2			; //
		BSF	bit_FlashB3			; //
		MOVLW	00BH				; Phase 11
		MOVWF	reg0_F08Phase			; //
		GOTO	sa_F08EXIT			; Exit function
; Switch to player A
sa_F08PH1020:	MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	0C0H				; No LED flashing
		ANDWF	reg_ProgCtrl3,F			; //
		MOVLW	003H				; Phase 3
		MOVWF	reg0_F08Phase			; //
		GOTO	sa_F08EXIT			; Exit function
;
sa_F08PH1050:	MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	000H				; Bx pointer = 0x00 -> offset = 0x08
		XORWF	reg0_CounterB,W			; //
		MOVLW	008H				; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F08PH1055			; //
		MOVLW	001H				; Bx pointer = 0x01 -> offset = 0x04
		XORWF	reg0_CounterB,W			; //
		MOVLW	004H				; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F08PH1055			; //
		MOVLW	000H				; Bx pointer = 0x02 -> offset = 0x00
;
sa_F08PH1055:	ADDWF	reg0_MovePtrB,W			; Piece B1 table pointer
		MOVWF	reg_Temp1			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece B1 from table
		ADDWF	reg0_PieceB1,W			; Calculate new position of piece B1
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceB1W			; //
		MOVLW	00CH				; Piece B2 table offset
		ADDWF	reg_Temp1,F			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece B2 from table
		ADDWF	reg0_PieceB2,W			; Calculate new position of piece B2
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceB2W			; //
		MOVLW	00CH				; Piece B3 table offset
		ADDWF	reg_Temp1,F			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece B3 from table
		ADDWF	reg0_PieceB3,W			; Calculate new position of piece B3
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceB3W			; //
		INCF	reg0_MovePtrB,F			; Increment move B pointer
		MOVLW	004H				; // value range 0x00 - 0x03
		XORWF	reg0_MovePtrB,W			; //
		BTFSC	STATUS,Z			; //
		CLRF	reg0_MovePtrB			; //
		MOVF	reg0_PieceA1,W			; Important before collision check !!!
		MOVWF	reg0_PieceA1W			; // Move figure A into working register
		MOVF	reg0_PieceA2,W			; //
		MOVWF	reg0_PieceA2W			; //
		MOVF	reg0_PieceA3,W			; //
		MOVWF	reg0_PieceA3W			; //
		mCALL01	sub_Collision			; Collision check
		BTFSC	STATUS,Z			; Collision detected ?
		GOTO	sa_F08PH1050			; // YES ! - search for valid move
		MOVF	reg0_PieceB1W,W			; // NO ! - update display
		MOVWF	reg0_DispB1			; //
		MOVF	reg0_PieceB2W,W			; //
		MOVWF	reg0_DispB2			; //
		MOVF	reg0_PieceB3W,W			; //
		MOVWF	reg0_DispB3			; //
		DECF	reg0_F08Phase,F			; Phase 9
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE11:	BTFSS	bit_Key				; All keys released ?
		CLRF	reg0_F08Phase			; // YES ! - phase 0
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
		mPRGMEMPAGE1				; --> PROGRAM MEMORY PAGE 1
		GOTO	00800H				; //
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; General subroutines within memory page 0
;
; Check if selected piece Ax is movable
; Piece Ax movable --> Z = 0, piece Ax not movable --> Z = 1
sub_AMOVABLE
sa_AMOV00:	MOVLW	000H				; Ax pointer = 0x00 -> offset = 0x08
		XORWF	reg0_CounterA,W			; //
		MOVLW	008H				; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_AMOV10			; //
		MOVLW	001H				; Ax pointer = 0x01 -> offset = 0x04
		XORWF	reg0_CounterA,W			; //
		MOVLW	004H				; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_AMOV10			; //
		MOVLW	000H				; Ax pointer = 0x02 -> offset = 0x00
;
sa_AMOV10:	ADDWF	reg0_MovePtrA,W			; Piece A1 table pointer
		MOVWF	reg_Temp1			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece A1 from table
		ADDWF	reg0_PieceA1,W			; Calculate new position of piece A1
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceA1W			; //
		MOVLW	00CH				; Piece A2 table offset
		ADDWF	reg_Temp1,F			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece A2 from table
		ADDWF	reg0_PieceA2,W			; Calculate new position of piece A2
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceA2W			; //
		MOVLW	00CH				; Piece A3 table offset
		ADDWF	reg_Temp1,F			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece A3 from table
		ADDWF	reg0_PieceA3,W			; Calculate new position of piece A3
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceA3W			; //
		MOVF	reg0_PieceB1,W			; Important before collision check !!!
		MOVWF	reg0_PieceB1W			; // Move figure B into working register
		MOVF	reg0_PieceB2,W			; //
		MOVWF	reg0_PieceB2W			; //
		MOVF	reg0_PieceB3,W			; //
		MOVWF	reg0_PieceB3W			; //
		mCALL01	sub_Collision			; Collision check
		BTFSS	STATUS,Z			; Collision detected ?
		RETURN					; // NO ! - piece Ax movable
		INCF	reg0_MovePtrA,F			; // YES ! - increment move A pointer
		MOVLW	004H				; // value range 0x00 - 0x03
		XORWF	reg0_MovePtrA,W			; //
		BTFSS	STATUS,Z			; Move A pointer overflow ?
		GOTO	sa_AMOV00			; // NO ! - check next move
		RETURN					; // YES ! - piece Ax not movable
;------------------------------------------------------------------------------------------------------------------------
; Check if selected piece Bx is movable
; Piece Bx movable --> Z = 0, piece Bx not movable --> Z = 1
sub_BMOVABLE
sa_BMOV00:	MOVLW	000H				; Bx pointer = 0x00 -> offset = 0x08
		XORWF	reg0_CounterB,W			; //
		MOVLW	008H				; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_BMOV10			; //
		MOVLW	001H				; Bx pointer = 0x01 -> offset = 0x04
		XORWF	reg0_CounterB,W			; //
		MOVLW	004H				; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_BMOV10			; //
		MOVLW	000H				; Bx pointer = 0x02 -> offset = 0x00
;
sa_BMOV10:	ADDWF	reg0_MovePtrB,W			; Piece B1 table pointer
		MOVWF	reg_Temp1			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece B1 from table
		ADDWF	reg0_PieceB1,W			; Calculate new position of piece B1
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceB1W			; //
		MOVLW	00CH				; Piece B2 table offset
		ADDWF	reg_Temp1,F			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece B2 from table
		ADDWF	reg0_PieceB2,W			; Calculate new position of piece B2
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceB2W			; //
		MOVLW	00CH				; Piece B3 table offset
		ADDWF	reg_Temp1,F			; //
		mCALL01	sa_F50TAB1			; Get move direction of piece B3 from table
		ADDWF	reg0_PieceB3,W			; Calculate new position of piece B3
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceB3W			; //
		MOVF	reg0_PieceA1,W			; Important before collision check !!!
		MOVWF	reg0_PieceA1W			; // Move figure B into working register
		MOVF	reg0_PieceA2,W			; //
		MOVWF	reg0_PieceA2W			; //
		MOVF	reg0_PieceA3,W			; //
		MOVWF	reg0_PieceA3W			; //
		mCALL01	sub_Collision			; Collision check
		BTFSS	STATUS,Z			; Collision detected ?
		RETURN					; // NO ! - piece Bx movable
		INCF	reg0_MovePtrB,F			; // YES ! - increment move B pointer
		MOVLW	004H				; // value range 0x00 - 0x03
		XORWF	reg0_MovePtrB,W			; //
		BTFSS	STATUS,Z			; Move B pointer overflow ?
		GOTO	sa_BMOV00			; // NO ! - check next move
		RETURN					; // YES ! - piece Bx not movable
;------------------------------------------------------------------------------------------------------------------------
; Figure A game over detection
; Game over --> Z = 1
sub_AGOVER:	MOVF	reg0_PieceA1W,W			; All X-values of figure A equal ?
		XORWF	reg0_PieceA2W,W			; //
		ANDLW	00FH				; // mask Y-value
		BTFSS	STATUS,Z			; //
		GOTO	sa_AGOVER00			; // NO ! - check Y-values
		MOVF	reg0_PieceA2W,W			; //
		XORWF	reg0_PieceA3W,W			; //
		ANDLW	00FH				; // mask Y-value
		BTFSS	STATUS,Z			; //
		GOTO	sa_AGOVER00			; // NO ! - check Y-values
		MOVF	reg0_PieceA1,W			; // YES ! - restore move working register
		MOVWF	reg0_PieceA1W			; //
		MOVF	reg0_PieceA2,W			; //
		MOVWF	reg0_PieceA2W			; //
		MOVF	reg0_PieceA3,W			; //
		MOVWF	reg0_PieceA3W			; //
		MOVF	reg0_PieceA1W,W			; Pieces A chain detected ?
		ADDWF	reg0_PieceA2W,W			; //
		ADDWF	reg0_PieceA3W,W			; //
		ANDLW	0F0H				; // mask X-value
		MOVWF	reg0_PieceA3W			; //
		MOVLW	030H				; //
		XORWF	reg0_PieceA3W,W			; //
		BTFSC	STATUS,Z			; //
		RETURN					; // YES ! - chain detected
		MOVLW	060H				; //
		XORWF	reg0_PieceA3W,W			; //
		BTFSC	STATUS,Z			; //
		RETURN					; // YES ! - chain detected
;
sa_AGOVER00:	MOVF	reg0_PieceA1W,W			; All Y-values of figure A equal ?
		XORWF	reg0_PieceA2W,W			; //
		ANDLW	0F0H				; // mask X-value
		BTFSS	STATUS,Z			; //
		RETURN					; // NO !
		MOVF	reg0_PieceA2W,W			; //
		XORWF	reg0_PieceA3W,W			; //
		ANDLW	0F0H				; // mask X-value
		BTFSS	STATUS,Z			; //
		RETURN					; // NO !
		MOVF	reg0_PieceA1,W			; // YES ! - restore move working register
		MOVWF	reg0_PieceA1W			; //
		MOVF	reg0_PieceA2,W			; //
		MOVWF	reg0_PieceA2W			; //
		MOVF	reg0_PieceA3,W			; //
		MOVWF	reg0_PieceA3W			; //
		MOVF	reg0_PieceA1W,W			; Pieces A chain detected ?
		ADDWF	reg0_PieceA2W,W			; //
		ADDWF	reg0_PieceA3W,W			; //
		ANDLW	00FH				; // mask Y-value
		MOVWF	reg0_PieceA3W			; //
		MOVLW	003H				; //
		XORWF	reg0_PieceA3W,W			; //
		BTFSC	STATUS,Z			; //
		RETURN					; // YES ! - chain detected
		MOVLW	006H				; //
		XORWF	reg0_PieceA3W,W			; //
		RETURN					; //
;------------------------------------------------------------------------------------------------------------------------
; Figure B game over detection
; Game over --> Z = 1
sub_BGOVER:	MOVF	reg0_PieceB1W,W			; All X-values of figure B equal ?
		XORWF	reg0_PieceB2W,W			; //
		ANDLW	00FH				; // mask Y-value
		BTFSS	STATUS,Z			; //
		GOTO	sa_BGOVER00			; // NO ! - check Y-values
		MOVF	reg0_PieceB2W,W			; //
		XORWF	reg0_PieceB3W,W			; //
		ANDLW	00FH				; // mask Y-value
		BTFSS	STATUS,Z			; //
		GOTO	sa_BGOVER00			; // NO ! - check Y-values
		MOVF	reg0_PieceB1,W			; // YES ! - restore move working register
		MOVWF	reg0_PieceB1W			; //
		MOVF	reg0_PieceB2,W			; //
		MOVWF	reg0_PieceB2W			; //
		MOVF	reg0_PieceB3,W			; //
		MOVWF	reg0_PieceB3W			; //
		MOVF	reg0_PieceB1W,W			; Pieces B chain detected ?
		ADDWF	reg0_PieceB2W,W			; //
		ADDWF	reg0_PieceB3W,W			; //
		ANDLW	0F0H				; // mask X-value
		MOVWF	reg0_PieceB3W			; //
		MOVLW	030H				; //
		XORWF	reg0_PieceB3W,W			; //
		BTFSC	STATUS,Z			; //
		RETURN					; // YES ! - chain detected
		MOVLW	060H				; //
		XORWF	reg0_PieceB3W,W			; //
		BTFSC	STATUS,Z			; //
		RETURN					; // YES ! - chain detected
;
sa_BGOVER00:	MOVF	reg0_PieceB1W,W			; All Y-values of figure B equal ?
		XORWF	reg0_PieceB2W,W			; //
		ANDLW	0F0H				; // mask X-value
		BTFSS	STATUS,Z			; //
		RETURN					; // NO !
		MOVF	reg0_PieceB2W,W			; //
		XORWF	reg0_PieceB3W,W			; //
		ANDLW	0F0H				; // mask X-value
		BTFSS	STATUS,Z			; //
		RETURN					; // NO !
		MOVF	reg0_PieceB1,W			; // YES ! - restore move working register
		MOVWF	reg0_PieceB1W			; //
		MOVF	reg0_PieceB2,W			; //
		MOVWF	reg0_PieceB2W			; //
		MOVF	reg0_PieceB3,W			; //
		MOVWF	reg0_PieceB3W			; //
		MOVF	reg0_PieceB1W,W			; Pieces B chain detected ?
		ADDWF	reg0_PieceB2W,W			; //
		ADDWF	reg0_PieceB3W,W			; //
		ANDLW	00FH				; // mask Y-value
		MOVWF	reg0_PieceB3W			; //
		MOVLW	003H				; //
		XORWF	reg0_PieceB3W,W			; //
		BTFSC	STATUS,Z			; //
		RETURN					; // YES ! - chain detected
		MOVLW	006H				; //
		XORWF	reg0_PieceB3W,W			; //
		RETURN					; //
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;		*********************************
;		*   PROGRAM  MEMORY  PAGE  1	*
;		*********************************
;
		ORG	0800H
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 09: Game "Are you quick enough ?"
		BTFSS	bit_Function09			; Function active
		GOTO	sa_F09EXIT			; // NO ! - skip function
;
		MOVLW	high(sa_F09PHASE)		; Function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg0_F09Phase,W			; //
		ADDWF	PCL,F				; //
sa_F09PHASE:	GOTO	sa_F09PHASE0			; //
		GOTO	sa_F09PHASE1			; //
		GOTO	sa_F09PHASE2			; //
		GOTO	sa_F09PHASE3			; //
		GOTO	sa_F09PHASE4			; //
		GOTO	sa_F09PHASE5			; //
		GOTO	sa_F09PHASE6			; //
		GOTO	sa_F09PHASE7			; //
sa_F09PHEND:	GOTO	sa_F09PHASE8			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F09PHASE) - high(sa_F09PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
; Demo mode
sa_F09PHASE0:	MOVF	reg0_Random,W			; Ensures, that two consecutive pseudo-random values are
		ANDLW	003H				; // NEW !!!
		XORWF	reg0_GameRnd,W			; // different
		BTFSC	STATUS,Z			; //
		GOTO	sa_F09EXIT			; Exit function
		MOVF	reg0_Random,W			; Store new pseudo-random valu
		MOVWF	reg0_GameRnd			; //
;
		CALL	sa_F09TABLE1			; Get random figure
		MOVWF	reg0_LEDDriver			; //
		MOVLW	006H				; Initialize demo speed
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; //
		INCF	reg0_F09Phase,F			; Phase 1
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE1:	BTFSC	bit_Key				; Any key pressed ?
		GOTO	sa_F09PH101			; // YES ! - start game
		BTFSC	bit_LongDelay4			; Figure period expired
		CLRF	reg0_F09Phase			; // YES ! - phase 0
		GOTO	sa_F09EXIT			; // NO ! - exit function
; Select game level
sa_F09PH101:	CLRF	reg0_GameLevel			; Initialize game level
		MOVLW	002H				; Key code 0x02 ?
		XORWF	reg0_KeyCode,W			; //
		MOVLW	000H				; // game level 1
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_GameLevel			; // YES !
;
		MOVLW	010H				; Key code 0x10 ?
		XORWF	reg0_KeyCode,W			; //
		MOVLW	001H				; // game level 2
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_GameLevel			; // YES !
;
		MOVLW	008H				; Key code 0x08 ?
		XORWF	reg0_KeyCode,W			; //
		MOVLW	002H				; // game level 3
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_GameLevel			; // YES !
;
		MOVLW	004H				; Key code 0x04 ?
		XORWF	reg0_KeyCode,W			; //
		MOVLW	003H				; // game level 4
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_GameLevel			; // YES !
;
		INCF	reg0_F09Phase,F			; Phase 2
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
; Game mode
sa_F09PHASE2:	MOVLW	000H				; Get level figure
		CALL	sa_F09TABLE2			; //
		MOVWF	reg0_LEDDriver			; //
		MOVLW	005H				; Get speed
		CALL	sa_F09TABLE2			; //
		MOVWF	reg0_GameSpeed			; //
		MOVLW	00AH				; Get step count
		CALL	sa_F09TABLE2			; //
		MOVWF	reg0_GameStep			; //
		MOVLW	004H				; Request beep code 0x04
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	016H				; Initialize level figure duration for 1.5 sec
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; //
		INCF	reg0_F09Phase,F			; Phase 3
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE3:	BTFSC	bit_LongDelay4			; Level figure duration expired ?
		INCF	reg0_F09Phase,F			; // YES ! - phase 4
		GOTO	sa_F09EXIT			; // NO ! - exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE4:	MOVF	reg0_Random,W			; Ensures, that two consecutive pseudo-random values are
		XORWF	reg0_GameRnd,W			; // different
		BTFSC	STATUS,Z			; //
		GOTO	sa_F09EXIT			; Exit function
		MOVF	reg0_Random,W			; Store new pseudo-random value
		MOVWF	reg0_GameRnd			; //
;
		CALL	sa_F09TABLE1			; Get random figure
		MOVWF	reg0_LEDDriver			; //
		MOVF	reg0_GameSpeed,W		; Initialize figure duration
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; //
		MOVLW	009H				; Request beep code 0x09
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		INCF	reg0_F09Phase,F			; Phase 5
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE5:	BTFSC	bit_LongDelay4			; Figure duration expired ?
		GOTO	sa_F09PH501			; // YES !
		BTFSS	bit_Key				; Key released ?
		INCF	reg0_F09Phase,F			; // YES ! - phase 6
		GOTO	sa_F09EXIT			; Exit function
;
sa_F09PH501:	MOVLW	0FFH				; Time-out
		MOVWF	reg0_LEDDriver			; //
;		MOVLW	008H				; Request beep code 0x08
		MOVLW	00BH				; Request beep code 0x0B
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	007H				; Phase 7
		MOVWF	reg0_F09Phase			; //
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE6:	BTFSC	bit_LongDelay4			; Figure duration expired ?
		GOTO	sa_F09PH601			; // YES !
		BTFSC	bit_Key				; Key pressed ?
		GOTO	sa_F09PH602			; // YES !
		GOTO	sa_F09EXIT			; // NO ! - exit function
;
sa_F09PH601:	MOVLW	0FFH				; Time-out
		MOVWF	reg0_LEDDriver			; //
;		MOVLW	008H				; Request beep code 0x08
		MOVLW	00BH				; Request beep code 0x0B
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		INCF	reg0_F09Phase,F			; Phase 7
		GOTO	sa_F09EXIT			; Exit function
;
sa_F09PH602:	MOVLW	004H				; Key code 0x04 ?
		XORWF	reg0_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH603			; // NO !
		MOVLW	0C0H				; Key & figure matching ?
		XORWF	reg0_LEDDriver,W		; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH601			; // NO !
		GOTO	sa_F09PH610			; // YES !
;
sa_F09PH603:	MOVLW	002H				; Key code 0x02 ?
		XORWF	reg0_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH604			; // NO !
		MOVLW	003H				; Key & figure matching ?
		XORWF	reg0_LEDDriver,W		; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH601			; // NO !
		GOTO	sa_F09PH610			; // YES !
;
sa_F09PH604:	MOVLW	008H				; Key code 0x08 ?
		XORWF	reg0_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH605			; // NO !
		MOVLW	030H				; Key & figure matching ?
		XORWF	reg0_LEDDriver,W		; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH601			; // NO !
		GOTO	sa_F09PH610			; // YES !
;
sa_F09PH605:	MOVLW	010H				; Key code 0x10 ?
		XORWF	reg0_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH601			; // NO !
		MOVLW	00CH				; Key & figure matching ?
		XORWF	reg0_LEDDriver,W		; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH601			; // NO !
;		GOTO	sa_F09PH610			; // YES !
;
sa_F09PH610:	MOVLW	008H				; Phase 8
		MOVWF	reg0_F09Phase			; //
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE7:	BTFSC	bit_Key				; Key released ?
		GOTO	sa_F09EXIT			; // NO ! - exit function
		BTFSS	bit_Beep			; Beep finished ?
		CLRF	reg0_F09Phase			; // YES ! - restart function
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE8:	BTFSS	bit_LongDelay4			; Figure duration expired ?
		GOTO	sa_F09EXIT			; // NO ! - exit function
;
		MOVLW	004H				; All steps done ? (phase 4)
		MOVWF	reg0_F09Phase			; //
		DECFSZ	reg0_GameStep,F			; //
		GOTO	sa_F09EXIT			; // NO ! - go to phase 4
;
		MOVLW	con_Game1MaxLevel		; Last game level ?
		XORWF	reg0_GameLevel,W		; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F09EXIT			; // YES ! - go to phase 4, no level change
		INCF	reg0_GameLevel,F		; Increment game level
		MOVLW	002H				; Phase 2
		MOVWF	reg0_F09Phase			; //
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09TABLE1:	MOVLW	high(sa_F09TABBEG1)		; Random figure selection
		MOVWF	PCLATH				; //
		MOVF	reg0_GameRnd,W			; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F09TABBEG1) - high(sa_F09TABEND1) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F09TABBEG1:	RETLW	003H				; Figure code 0x00
		RETLW	00CH				; Figure code 0x01
		RETLW	030H				; Figure code 0x02
sa_F09TABEND1:	RETLW	0C0H				; Figure code 0x03
;------------------------------------------------------------------------------------------------------------------------
; Table offset Wreg = 0x00 --> get level figure /  Wreg = 0x05 --> get speed /  Wreg = 0x0A --> get step count
sa_F09TABLE2:	MOVWF	reg_Temp1			; Get game parameters
		MOVLW	high(sa_F09TABBEG2)		; //
		MOVWF	PCLATH				; //
		MOVF	reg_Temp1,W			; //
		ADDWF	reg0_GameLevel,W		; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F09TABBEG2) - high(sa_F09TABEND2) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
; Level figure
sa_F09TABBEG2:	RETLW	020H				; Level 1 - figure
		RETLW	060H				; Level 2 - figure
		RETLW	070H				; Level 3 - figure
		RETLW	0F0H				; Level 4 - figure
		RETLW	0F8H				; Level 5 - figure
; Level speed
		RETLW	013H				; Level 1 - speed
		RETLW	00FH				; Level 2 - speed
		RETLW	00BH				; Level 3 - speed
		RETLW	008H				; Level 4 - speed
		RETLW	006H				; Level 5 - speed
; Level step count
		RETLW	00AH				; Level 1 - step count (10)
		RETLW	00AH				; Level 2 - step count (10)
		RETLW	014H				; Level 3 - step count (20)
		RETLW	01EH				; Level 4 - step count (30)
sa_F09TABEND2:	RETLW	005H				; Level 5 - step count (unlimited)
;------------------------------------------------------------------------------------------------------------------------
sa_F09EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 10: Game "Memory game"
		BTFSS	bit_Function10			; Function active
		GOTO	sa_F10EXIT			; // NO ! - skip function
;
		MOVLW	high(sa_F10PHASE)		; Function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg0_F10Phase,W			; //
		ADDWF	PCL,F				; //
sa_F10PHASE:	GOTO	sa_F10PHASE00			; //
		GOTO	sa_F10PHASE01			; //
		GOTO	sa_F10PHASE02			; //
		GOTO	sa_F10PHASE03			; //
		GOTO	sa_F10PHASE04			; //
		GOTO	sa_F10PHASE05			; //
		GOTO	sa_F10PHASE06			; //
		GOTO	sa_F10PHASE07			; //
		GOTO	sa_F10PHASE08			; //
		GOTO	sa_F10PHASE09			; //
		GOTO	sa_F10PHASE10			; //
		GOTO	sa_F10PHASE11			; //
sa_F10PHEND:	GOTO	sa_F10PHASE12			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F10PHASE) - high(sa_F10PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
; Demo mode
sa_F10PHASE00:	CALL	sa_F10TABLE1			; Get random figure
		MOVWF	reg0_LEDDriver			; //
		MOVLW	007H				; Initialize demo figure duration
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; //
		INCF	reg0_F10Phase,F			; Phase 1
		GOTO	sa_F10EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F10PHASE01:	BTFSC	bit_Key				; Any key pressed ?
		GOTO	sa_F10PH0301			; // YES ! - start game (go directly to phase 3)
		BTFSC	bit_LongDelay4			; Figure period expired
		INCF	reg0_F10Phase,F			; // YES ! - phase 2
		GOTO	sa_F10EXIT			; // NO ! - exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F10PHASE02:	CLRF	reg0_LEDDriver			; Turn-off all LEDs
		MOVLW	004H				; Initialize demo figure duration
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; //
		INCF	reg0_F10Phase,F			; Phase 3
		GOTO	sa_F10EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F10PHASE03:	BTFSC	bit_Key				; Any key pressed ?
		GOTO	sa_F10PH0301			; // YES ! - start game
		BTFSC	bit_LongDelay4			; Figure period expired
		CLRF	reg0_F10Phase			; // YES ! - phase 0
		GOTO	sa_F10EXIT			; // NO ! - exit function
;
sa_F10PH0301:	CLRF	reg0_GameLevel			; Initialize game level
		MOVLW	002H				; Key code 0x02 ?
		XORWF	reg0_KeyCode,W			; //
		MOVLW	000H				; // game level 1
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_GameLevel			; // YES !
;
		MOVLW	010H				; Key code 0x10 ?
		XORWF	reg0_KeyCode,W			; //
		MOVLW	001H				; // game level 2
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_GameLevel			; // YES !
;
		MOVLW	008H				; Key code 0x08 ?
		XORWF	reg0_KeyCode,W			; //
		MOVLW	002H				; // game level 3
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_GameLevel			; // YES !
;
		MOVLW	004H				; Key code 0x04 ?
		XORWF	reg0_KeyCode,W			; //
		MOVLW	003H				; // game level 4
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_GameLevel			; // YES !
;
		MOVLW	004H				; Phase 4
		MOVWF	reg0_F10Phase			; //
		GOTO	sa_F10EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
; Game mode
sa_F10PHASE04:	MOVLW	000H				; Get level figure
		CALL	sa_F10TABLE2			; //
		MOVWF	reg0_LEDDriver			; //
		MOVLW	00EH				; Get repeat count
		CALL	sa_F10TABLE2			; //
		MOVWF	reg0_GameRepeat			; //
		MOVLW	004H				; Request beep code 0x04
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	016H				; Initialize level figure duration for 1.5 sec
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; //
		INCF	reg0_F10Phase,F			; Phase 5
		GOTO	sa_F10EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F10PHASE05:	BTFSS	bit_LongDelay4			; Level figure duration expired ?
		GOTO	sa_F10EXIT			; // NO ! - exit function
;
		MOVLW	007H				; Get step count
		CALL	sa_F10TABLE2			; //
		MOVWF	reg0_GameStep			; //
		CLRF	reg0_GameBuf1			; Clear buffer
		CLRF	reg0_GameBuf2			; //
		INCF	reg0_F10Phase,F			; Phase 6
		GOTO	sa_F10EXIT			; // NO ! - exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F10PHASE06:	BCF	STATUS,C			; Store random value the buffer
		RLF	reg0_GameBuf1,F			; //
		RLF	reg0_GameBuf2,F			; //
		RLF	reg0_GameBuf1,F			; //
		RLF	reg0_GameBuf2,F			; //
		MOVF	reg0_Random,W			; //
		IORWF	reg0_GameBuf1,F			; //
		CALL	sa_F10TABLE1			; Get random figure
		MOVWF	reg0_LEDDriver			; //
		MOVLW	007H				; Initialize figure duration
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; //
		MOVLW	009H				; Request beep code 0x09
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		INCF	reg0_F10Phase,F			; Phase 7
		GOTO	sa_F10EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F10PHASE07:	BTFSC	bit_LongDelay4			; Figure period expired ?
		INCF	reg0_F10Phase,F			; // YES ! - phase 8
		GOTO	sa_F10EXIT			; // NO ! - exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F10PHASE08:	CLRF	reg0_LEDDriver			; Turn-off all LEDs
		MOVLW	004H				; Initialize figure duration
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; //
		INCF	reg0_F10Phase,F			; Phase 9
		GOTO	sa_F10EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F10PHASE09:	BTFSS	bit_LongDelay4			; Figure period expired ?
		GOTO	sa_F10EXIT			; // NO ! - exit function
		DECFSZ	reg0_GameStep,F			; All steps done ?
		GOTO	sa_F10PH0990			; // NO !
;
		MOVLW	007H				; // YES ! - get step count
		CALL	sa_F10TABLE2			; //
		MOVWF	reg0_GameStep			; //
;
		MOVLW	008H				; Align buffer
		SUBWF	reg0_GameStep,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F10PH0902			; //
		XORLW	0FFH				; // two's complement
		ADDLW	001H				; //
		MOVWF	reg_Temp1			; //
sa_F10PH0901:	RLF	reg0_GameBuf1,F			; //
		RLF	reg0_GameBuf2,F			; //
		RLF	reg0_GameBuf1,F			; //
		RLF	reg0_GameBuf2,F			; //
		DECFSZ	reg_Temp1,F			; //
		GOTO	sa_F10PH0901			; //
sa_F10PH0902:	MOVLW	00AH				; Phase 10
		MOVWF	reg0_F10Phase			; //
		GOTO	sa_F10EXIT			; Exit function
;
sa_F10PH0990:	MOVLW	006H				; Phase 6
		MOVWF	reg0_F10Phase			; //
		GOTO	sa_F10EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F10PHASE10:	BTFSS	bit_Key				; Key pressed ?
		GOTO	sa_F10EXIT			; // NO ! - exit function
;
		RLF	reg0_GameBuf1,F			; // YES ! - get random code from buffer
		RLF	reg0_GameBuf2,F			; //
		RLF	reg_Temp1,F			; //
		RLF	reg0_GameBuf1,F			; //
		RLF	reg0_GameBuf2,F			; //
		RLF	reg_Temp1,F			; //
		MOVLW	003H				; //
		ANDWF	reg_Temp1,F			; //
;		
sa_F10PH1010:	MOVLW	004H				; Key code 0x04 ?
		XORWF	reg0_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F10PH1011			; // NO !
		MOVLW	003H				; Key & buffer matching ?
		XORWF	reg_Temp1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F10PH1090			; // NO !
		MOVLW	0C0H				; // YES ! - display match figure
		GOTO	sa_F10PH1014			; //
;
sa_F10PH1011:	MOVLW	002H				; Key code 0x02 ?
		XORWF	reg0_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F10PH1012			; // NO !
		MOVLW	000H				; Key & buffer matching ?
		XORWF	reg_Temp1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F10PH1090			; // NO !
		MOVLW	003H				; // YES ! - display match figure
		GOTO	sa_F10PH1014			; //
;
sa_F10PH1012:	MOVLW	008H				; Key code 0x08 ?
		XORWF	reg0_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F10PH1013			; // NO !
		MOVLW	002H				; Key & buffer matching ?
		XORWF	reg_Temp1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F10PH1090			; // NO !
		MOVLW	030H				; // YES ! - display match figure
		GOTO	sa_F10PH1014			; //
;
sa_F10PH1013:	MOVLW	010H				; Key code 0x10 ?
		XORWF	reg0_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F10PH1090			; // NO !
		MOVLW	001H				; Key & buffer matching ?
		XORWF	reg_Temp1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F10PH1090			; // NO !
		MOVLW	00CH				; // YES ! - display match figure
;		GOTO	sa_F10PH1014			; //
;
sa_F10PH1014:	MOVWF	reg0_LEDDriver			; Display match figure
		MOVLW	009H				; Request beep code 0x09
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		INCF	reg0_F10Phase,F			; Phase 11
		GOTO	sa_F10EXIT			; Exit function
;
sa_F10PH1090:	MOVLW	0FFH				; Key & buffer mismatch figure
		MOVWF	reg0_LEDDriver			; //
;		MOVLW	008H				; Request beep code 0x08
		MOVLW	00BH				; Request beep code 0x0B
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	00CH				; Phase 12
		MOVWF	reg0_F10Phase			; //
		GOTO	sa_F10EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F10PHASE11:	BTFSC	bit_Key				; Key released ?
		GOTO	sa_F10EXIT			; // NO ! - exit function
		CLRF	reg0_LEDDriver			; // YES ! - clear match figure
		DECF	reg0_F10Phase,F			; All steps processed ?
		DECFSZ	reg0_GameStep,F			; //
		GOTO	sa_F10EXIT			; // NO ! - go to phase 10

		MOVLW	009H				; // YES ! - delay before next sequence starts
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; //
;
		MOVLW	005H				; Level finished ? (Set phase 5)
		MOVWF	reg0_F10Phase			; //
		DECFSZ	reg0_GameRepeat,F		; //
		GOTO	sa_F10EXIT			; // NO ! - go to phase 5
;
		MOVLW	con_Game2MaxLevel		; Last game level ?
		XORWF	reg0_GameLevel,W		; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F10EXIT			; // YES ! - go to phase 5 (no level change)
		INCF	reg0_GameLevel,F		; // NO ! - increment game level
		MOVLW	004H				; Phase 4
		MOVWF	reg0_F10Phase			; //
		GOTO	sa_F10EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F10PHASE12:	BTFSC	bit_Key				; Key released ?
		GOTO	sa_F10EXIT			; // NO ! - exit function
		BTFSS	bit_Beep			; Beep finished ?
		CLRF	reg0_F10Phase			; // YES ! - restart function
		GOTO	sa_F10EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F10TABLE1:	MOVLW	high(sa_F10TABBEG1)		; Random figure selection
		MOVWF	PCLATH				; //
		MOVF	reg0_Random,W			; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F10TABBEG1) - high(sa_F10TABEND1) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F10TABBEG1:	RETLW	003H				; Figure code 0x00
		RETLW	00CH				; Figure code 0x01
		RETLW	030H				; Figure code 0x02
sa_F10TABEND1:	RETLW	0C0H				; Figure code 0x03
;------------------------------------------------------------------------------------------------------------------------
; Table offset Wreg = 0x00 --> get level figure /  Wreg = 0x07 --> get step count /  Wreg = 0x0E --> get repeat count
sa_F10TABLE2:	MOVWF	reg_Temp1			; Get game parameters
		MOVLW	high(sa_F10TABBEG2)		; //
		MOVWF	PCLATH				; //
		MOVF	reg_Temp1,W			; //
		ADDWF	reg0_GameLevel,W		; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F10TABBEG2) - high(sa_F10TABEND2) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
; Level figure
sa_F10TABBEG2:	RETLW	020H				; Level 1 - figure
		RETLW	060H				; Level 2 - figure
		RETLW	070H				; Level 3 - figure
		RETLW	0F0H				; Level 4 - figure
		RETLW	0F8H				; Level 5 - figure
		RETLW	0F9H				; Level 6 - figure
		RETLW	0FDH				; Level 7 - figure
; Level step count
		RETLW	002H				; Level 1 - step count
		RETLW	003H				; Level 2 - step count
		RETLW	004H				; Level 3 - step count
		RETLW	005H				; Level 4 - step count
		RETLW	006H				; Level 5 - step count
		RETLW	007H				; Level 6 - step count
		RETLW	008H				; Level 7 - step count
; Level repeat count
		RETLW	003H				; Level 1 - repeat count
		RETLW	003H				; Level 2 - repeat count
		RETLW	003H				; Level 3 - repeat count
		RETLW	003H				; Level 4 - repeat count
		RETLW	003H				; Level 5 - repeat count
		RETLW	003H				; Level 6 - repeat count
sa_F10TABEND2:	RETLW	003H				; Level 7 - repeat count (unlimited)
;------------------------------------------------------------------------------------------------------------------------
sa_F10EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 11: LED controller @ game no. 4 / 5
; The function converts content of reg0_LEDDriver (used within CreativeBoard) into content of reg0_LEDDriver1 - 4.
; The use of this conversion allows not to modify game functions 09 & 10.
;
		BTFSS	bit_Function11			; Function requested ?
		GOTO	sa_F11EXIT			; // NO ! - skip function
;
		MOVLW	003H				; Figure code 0x03 ?
		XORWF	reg0_LEDDriver,W		; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F1100			; // NO !
		MOVLW	000H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	080H				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	080H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	040H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F11EXIT			; Exit function
;
sa_F1100:	MOVLW	00CH				; Figure code 0x0C ?
		XORWF	reg0_LEDDriver,W		; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F1110			; // NO !
		MOVLW	008H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	004H				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	000H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	008H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F11EXIT			; Exit function
;
sa_F1110:	MOVLW	030H				; Figure code 0x30 ?
		XORWF	reg0_LEDDriver,W		; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F1120			; // NO !
		MOVLW	001H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	002H				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	000H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	001H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F11EXIT			; Exit function
;
sa_F1120:	MOVLW	0C0H				; Figure code 0xC0 ?
		XORWF	reg0_LEDDriver,W		; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F1130			; // NO !
		MOVLW	000H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	010H				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	010H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	020H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F11EXIT			; Exit function
;
sa_F1130:	MOVLW	000H				; Figure code 0x00 ?
		XORWF	reg0_LEDDriver,W		; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F1140			; // NO !
		CLRF	reg0_LEDDriver1			; // YES !
		CLRF	reg0_LEDDriver2			; //
		CLRF	reg0_LEDDriver3			; //
		CLRF	reg0_LEDDriver4			; //
		GOTO	sa_F11EXIT			; Exit function
;
sa_F1140:	MOVLW	0FFH				; Figure code 0xFF ?
		XORWF	reg0_LEDDriver,W		; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F1150			; // NO !
		MOVLW	055H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	000H				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	002H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	000H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F11EXIT			; Exit function
;
sa_F1150:	MOVF	reg0_LEDDriver,W		; Other figure codes
		MOVWF	reg0_LEDDriver2			; //
		CLRF	reg0_LEDDriver1			; //
		CLRF	reg0_LEDDriver3			; //
		CLRF	reg0_LEDDriver4			; //
		GOTO	sa_F11EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F11EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 12: Electronic dice A
;
		BTFSS	bit_Function12			; Function requested ?
		GOTO	sa_F12EXIT			; // NO ! - skip function
;
		MOVLW	high(sa_F12PHASE)		; // YES ! - function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg0_F12Phase,W			; //
		ADDWF	PCL,F				; //
sa_F12PHASE:	GOTO	sa_F12PHASE0			; //
		GOTO	sa_F12PHASE1			; //
sa_F12PHEND:	GOTO	sa_F12PHASE2			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F12PHASE) - high(sa_F12PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F12PHASE0:	BTFSS	bit_Key				; Any key pressed ?
		GOTO	sa_F12EXIT			; // NO ! - exit function
		MOVLW	001H				; // YES ! - beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		INCF	reg0_F12Phase,F			; Phase 1
		GOTO	sa_F12EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F12PHASE1:	MOVLW	007H				; Random value between 0x00 - 0x05 ?
		ANDWF	reg0_Random,W			; //
		MOVWF	reg_Temp1			; Store random value
		SUBLW	005H				; //
		BTFSS	STATUS,C			; //
		GOTO	sa_F12EXIT			; // NO ! - exit function
; number 1
		MOVLW	000H				; Random value equal 0x00 ?
		XORWF	reg_Temp1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F12PH0100			; // NO !
		MOVLW	020H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	08FH				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	000H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	088H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F12PH0199			; //
; number 2
sa_F12PH0100:	MOVLW	001H				; Random value equal 0x01 ?
		XORWF	reg_Temp1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F12PH0110			; // NO !
		MOVLW	000H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	08FH				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	041H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	088H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F12PH0199			; //
; number 3
sa_F12PH0110:	MOVLW	002H				; Random value equal 0x02 ?
		XORWF	reg_Temp1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F12PH0120			; // NO !
		MOVLW	020H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	08FH				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	041H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	088H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F12PH0199			; //
; number 4
sa_F12PH0120:	MOVLW	003H				; Random value equal 0x03 ?
		XORWF	reg_Temp1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F12PH0130			; // NO !
		MOVLW	000H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	08FH				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	055H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	088H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F12PH0199			; //
; number 5
sa_F12PH0130:	MOVLW	004H				; Random value equal 0x04 ?
		XORWF	reg_Temp1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F12PH0140			; // NO !
		MOVLW	020H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	08FH				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	055H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	088H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F12PH0199			; //
; number 6
sa_F12PH0140:	MOVLW	005H				; Random value equal 0x05 ?
		XORWF	reg_Temp1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F12PH0199			; // NO !
		MOVLW	000H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	08FH				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	077H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	088H				; //
		MOVWF	reg0_LEDDriver4			; //
;
sa_F12PH0199:	INCF	reg0_F12Phase,F			; Phase 2
		GOTO	sa_F12EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F12PHASE2:	BTFSS	bit_Key				; All keys released ?
		CLRF	reg0_F12Phase			; // YES ! - phase 0
		GOTO	sa_F12EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F12EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 13: Electronic dice B
;
		BTFSS	bit_Function13			; Function requested ?
		GOTO	sa_F13EXIT			; // NO ! - skip function
;
		MOVLW	high(sa_F13PHASE)		; // YES ! - function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg0_F13Phase,W			; //
		ADDWF	PCL,F				; //
sa_F13PHASE:	GOTO	sa_F13PHASE0			; //
sa_F13PHEND:	GOTO	sa_F13PHASE1			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F13PHASE) - high(sa_F13PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F13PHASE0:	BTFSS	bit_Key				; Any key pressed ?
		GOTO	sa_F13EXIT			; // NO ! - exit function
		MOVLW	001H				; // YES ! - beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	000H				; Empty figure
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	08FH				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	000H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	088H				; //
		MOVWF	reg0_LEDDriver4			; //
		CLRF	reg0_DiceCnt			; Initialize dice counter
		INCF	reg0_F13Phase,F			; Phase 1
		GOTO	sa_F13EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F13PHASE1:	INCF	reg0_DiceCnt,F			; Dice counter value range 0x00 - 0x05
		MOVLW	006H				; //
		XORWF	reg0_DiceCnt,W			; //
		BTFSC	STATUS,Z			; //
		CLRF	reg0_DiceCnt			; //
;
		BTFSC	bit_Key				; All keys released ?
		GOTO	sa_F13EXIT			; // NO ! - exit function
; number 1
		MOVLW	000H				; Random value equal 0x00 ?
		XORWF	reg0_DiceCnt,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F13PH0100			; // NO !
		MOVLW	020H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	08FH				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	000H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	088H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F13PH0199			; //
; number 2
sa_F13PH0100:	MOVLW	001H				; Random value equal 0x01 ?
		XORWF	reg0_DiceCnt,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F13PH0110			; // NO !
		MOVLW	000H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	08FH				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	041H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	088H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F13PH0199			; //
; number 3
sa_F13PH0110:	MOVLW	002H				; Random value equal 0x02 ?
		XORWF	reg0_DiceCnt,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F13PH0120			; // NO !
		MOVLW	020H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	08FH				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	041H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	088H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F13PH0199			; //
; number 4
sa_F13PH0120:	MOVLW	003H				; Random value equal 0x03 ?
		XORWF	reg0_DiceCnt,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F13PH0130			; // NO !
		MOVLW	000H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	08FH				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	055H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	088H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F13PH0199			; //
; number 5
sa_F13PH0130:	MOVLW	004H				; Random value equal 0x04 ?
		XORWF	reg0_DiceCnt,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F13PH0140			; // NO !
		MOVLW	020H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	08FH				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	055H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	088H				; //
		MOVWF	reg0_LEDDriver4			; //
		GOTO	sa_F13PH0199			; //
; number 6
sa_F13PH0140:	MOVLW	005H				; Random value equal 0x05 ?
		XORWF	reg0_DiceCnt,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F13PH0199			; // NO !
		MOVLW	000H				; // YES !
		MOVWF	reg0_LEDDriver1			; //
		MOVLW	08FH				; //
		MOVWF	reg0_LEDDriver2			; //
		MOVLW	077H				; //
		MOVWF	reg0_LEDDriver3			; //
		MOVLW	088H				; //
		MOVWF	reg0_LEDDriver4			; //
;
sa_F13PH0199:	CLRF	reg0_F13Phase			; Phase 0
		GOTO	sa_F13EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F13EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 14: Game no. 6 - "The falling stars"
		BTFSS	bit_Function14			; Function active
		GOTO	sa_F14EXIT			; // NO ! - skip function
;
		MOVLW	high(sa_F14PHASE)		; Function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg0_F14Phase,W			; //
		ADDWF	PCL,F				; //
sa_F14PHASE:	GOTO	sa_F14PHASE00			; //
		GOTO	sa_F14PHASE01			; //
		GOTO	sa_F14PHASE02			; //
		GOTO	sa_F14PHASE03			; //
		GOTO	sa_F14PHASE04			; //
		GOTO	sa_F14PHASE05			; //
		GOTO	sa_F14PHASE06			; //
		GOTO	sa_F14PHASE07			; //
sa_F14PHEND:	GOTO	sa_F14PHASE08			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F14PHASE) - high(sa_F14PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
; Demo mode
sa_F14PHASE00:	CLRF	reg0_Column0			; Clear display
		CLRF	reg0_Column1			; //
		CLRF	reg0_Column2			; //
		CLRF	reg0_Column3			; //
		CLRF	reg0_LEDDriver1			; //
		CLRF	reg0_LEDDriver2			; //
		CLRF	reg0_LEDDriver3			; //
		CLRF	reg0_LEDDriver4			; //
		INCF	reg0_F14Phase,F			; Phase 1
		GOTO	sa_F14EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F14PHASE01:	MOVLW	007H				; Get new falling stone (pseudo-random column)
		ANDWF	reg0_Random,F			; //
		MOVLW	000H				; //
		XORWF	reg0_Random,W			; //
		BTFSC	STATUS,Z			; //
		BSF	reg0_Column0,000H		; //
		MOVLW	001H				; //
		XORWF	reg0_Random,W			; //
		BTFSC	STATUS,Z			; //
		BSF	reg0_Column1,000H		; //
		MOVLW	002H				; //
		XORWF	reg0_Random,W			; //
		BTFSC	STATUS,Z			; //
		BSF	reg0_Column2,000H		; //
		MOVLW	003H				; //
		XORWF	reg0_Random,W			; //
		BTFSC	STATUS,Z			; //
		BSF	reg0_Column3,000H		; //
		BCF	STATUS,C			; Display scrolling down (falling stone effect)
		RLF	reg0_Column0,F			; //
		BCF	STATUS,C			; //
		RLF	reg0_Column1,F			; //
		BCF	STATUS,C			; //
		RLF	reg0_Column2,F			; //
		BCF	STATUS,C			; //
		RLF	reg0_Column3,F			; //
		CALL	sub_F14DISP			; Refresh display
		MOVLW	007H				; Initialize demo figure duration
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; //
		INCF	reg0_F14Phase,F			; Phase 2
		GOTO	sa_F14EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F14PHASE02:	BTFSC	bit_Key				; Any key pressed ?
		GOTO	sa_F14PH0200			; // YES !
		BTFSC	bit_LongDelay4			; Delay expired ?
		DECF	reg0_F14Phase,F			; // YES ! - phase 1
		GOTO	sa_F14EXIT			; // NO ! - exit function
;
sa_F14PH0200:	CLRF	reg0_GameLevel			; Initialize game level
		MOVLW	002H				; Key code 0x02 ?
		XORWF	reg0_KeyCode,W			; //
		MOVLW	000H				; // game level 1
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_GameLevel			; // YES !
;
		MOVLW	010H				; Key code 0x10 ?
		XORWF	reg0_KeyCode,W			; //
		MOVLW	001H				; // game level 2
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_GameLevel			; // YES !
;
		MOVLW	008H				; Key code 0x08 ?
		XORWF	reg0_KeyCode,W			; //
		MOVLW	002H				; // game level 3
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_GameLevel			; // YES !
;
		MOVLW	004H				; Key code 0x04 ?
		XORWF	reg0_KeyCode,W			; //
		MOVLW	003H				; // game level 4
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_GameLevel			; // YES !
;
		MOVLW	003H				; Initialize life counter
		MOVWF	reg0_GameRepeat			; //
		INCF	reg0_F14Phase,F			; Phase 3
		GOTO	sa_F14EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
; Game mode
sa_F14PHASE03:	MOVLW	000H				; Get level figure
		CALL	sa_F14TABLE			; //
		MOVWF	reg0_LEDDriver2			; Display level figure
		CLRF	reg0_LEDDriver1			; //
		CLRF	reg0_LEDDriver3			; //
		CLRF	reg0_LEDDriver4			; //
		MOVLW	007H				; Get game speed
		CALL	sa_F14TABLE			; //
		MOVWF	reg0_GameSpeed			; //
		MOVLW	00EH				; Get level step count
		CALL	sa_F14TABLE			; //
		MOVWF	reg0_GameStep			; //
		MOVLW	004H				; Request beep code 0x04
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	016H				; Initialize level figure duration for 1.5 sec
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; //
		INCF	reg0_F14Phase,F			; Phase 5
		GOTO	sa_F14EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F14PHASE04:	BTFSS	bit_LongDelay4			; Delay expired ?
		GOTO	sa_F14EXIT			; // NO ! - exit function
		CLRF	reg0_Column0			; // YES ! - clear display
		CLRF	reg0_Column1			; //
		CLRF	reg0_Column2			; //
		CLRF	reg0_Column3			; //
		CLRF	reg0_LEDDriver1			; //
		CLRF	reg0_LEDDriver2			; //
		CLRF	reg0_LEDDriver3			; //
		CLRF	reg0_LEDDriver4			; //
		BSF	bit_Function15			; Run function 15 - red point moving
		CLRF	reg0_F15Phase			; //
		MOVLW	004H				; //
		MOVWF	reg0_G6Row			; //
		INCF	reg0_F14Phase,F			; Phase 5
		GOTO	sa_F14EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F14PHASE05:	MOVLW	007H				; Get new falling stone (pseudo-random column)
		ANDWF	reg0_Random,F			; //
		MOVLW	000H				; //
		XORWF	reg0_Random,W			; //
		BTFSC	STATUS,Z			; //
		BSF	reg0_Column0,000H		; //
		MOVLW	001H				; //
		XORWF	reg0_Random,W			; //
		BTFSC	STATUS,Z			; //
		BSF	reg0_Column1,000H		; //
		MOVLW	002H				; //
		XORWF	reg0_Random,W			; //
		BTFSC	STATUS,Z			; //
		BSF	reg0_Column2,000H		; //
		MOVLW	003H				; //
		XORWF	reg0_Random,W			; //
		BTFSC	STATUS,Z			; //
		BSF	reg0_Column3,000H		; //
		BCF	STATUS,C			; Display scrolling down (falling stone effect)
		RLF	reg0_Column0,F			; //
		BCF	STATUS,C			; //
		RLF	reg0_Column1,F			; //
		BCF	STATUS,C			; //
		RLF	reg0_Column2,F			; //
		BCF	STATUS,C			; //
		RLF	reg0_Column3,F			; //
		CALL	sub_F14DISP			; Refresh display
;
		BTFSC	reg0_Column0,004H		; Collision check with red point
		GOTO	sa_F14PH0500			; //
		BTFSC	reg0_Column1,004H		; //
		GOTO	sa_F14PH0510			; //
		BTFSC	reg0_Column2,004H		; //
		GOTO	sa_F14PH0520			; //
		BTFSC	reg0_Column3,004H		; //
		GOTO	sa_F14PH0530			; //
		GOTO	sa_F14PH0595			; // No collision check
;
sa_F14PH0500:	BTFSS	reg0_G6Row,000H			; Collision check column 0
		GOTO	sa_F14PH0599			; No collision -> decrement life counter / game over
		BCF	reg0_Column0,004H		; //
		GOTO	sa_F14PH0590			; //
sa_F14PH0510:	BTFSS	reg0_G6Row,001H			; Collision check column 1
		GOTO	sa_F14PH0599			; No collision -> decrement life counter / game over
		BCF	reg0_Column1,004H		; //
		GOTO	sa_F14PH0590			; //
sa_F14PH0520:	BTFSS	reg0_G6Row,002H			; Collision check column 2
		GOTO	sa_F14PH0599			; No collision -> decrement life counter / game over
		BCF	reg0_Column2,004H		; //
		GOTO	sa_F14PH0590			; //
sa_F14PH0530:	BTFSS	reg0_G6Row,003H			; Collision check column 3
		GOTO	sa_F14PH0599			; No collision -> decrement life counter / game over
		BCF	reg0_Column3,004H		; //
		GOTO	sa_F14PH0590			; //
;
sa_F14PH0590:	DECFSZ	reg0_GameStep,F			; All steps done ?
		GOTO	sa_F14PH0592			; // NO !
		INCF	reg0_GameLevel,F		; // YES ! - change to next game level
		MOVLW	007H				; //
		XORWF	reg0_GameLevel,W		; //
		MOVLW	006H				; //
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_GameLevel			; //
		INCF	reg0_GameRepeat,F		; BONUS - increment life counter by one
		MOVLW	003H				; Phase 3
		MOVWF	reg0_F14Phase			; //
		GOTO	sa_F14EXIT			; Exit function
;
sa_F14PH0592:	CALL	sub_F14DISP			; Refresh display
		MOVLW	001H				; Beep OK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
;
sa_F14PH0595:	MOVF	reg0_GameSpeed,W		; Initialize figure duration
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; //
		INCF	reg0_F14Phase,F			; Phase 6
		GOTO	sa_F14EXIT			; Exit function
;
sa_F14PH0599:	MOVLW	007H				; Beep NOK
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		BCF	bit_Function15			; Stop function 15
		MOVLW	016H				; Initialize delay for 1.5 sec
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; //
		MOVLW	007H				; Phase 7
		MOVWF	reg0_F14Phase			; //
		GOTO	sa_F14EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F14PHASE06:	BTFSS	bit_LongDelay4			; Delay expired ?
		GOTO	sa_F14EXIT			; // NO ! - exit function
		DECF	reg0_F14Phase,F			; Phase 5
		GOTO	sa_F14EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F14PHASE07:	BTFSS	bit_LongDelay4			; Delay expired ?
		GOTO	sa_F14EXIT			; // NO ! - exit function
;
		DECFSZ	reg0_GameRepeat,F		; Decrement life counter - game over ?
		GOTO	sa_F14PH0710			; // NO ! - continue game
		MOVLW	00BH				; // YES ! - game over melody
		MOVWF	reg0_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	016H				; Initialize delay for 1.5 sec
		MOVWF	reg0_LongDelay4H		; //
		MOVLW	0FFH				; //
		MOVWF	reg0_LongDelay4L		; //
		BCF	bit_LongDelay4			; //
		INCF	reg0_F14Phase,F			; Phase 8
		GOTO	sa_F14EXIT			; Exit function
;
sa_F14PH0710:	MOVLW	004H				; Phase 4
		MOVWF	reg0_F14Phase			; //
		GOTO	sa_F14EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F14PHASE08:	BTFSC	bit_LongDelay4			; Delay expired ?
		CLRF	reg0_F14Phase			; // YES ! - phase 0
		GOTO	sa_F14EXIT			; // NO ! - exit function
;------------------------------------------------------------------------------------------------------------------------
; Table offset Wreg = 0x00 --> get level figure /  Wreg = 0x07 --> get game speed /  Wreg = 0x0E --> get step count
sa_F14TABLE:	MOVWF	reg_Temp1			; Get game parameters
		MOVLW	high(sa_F14TABBEG)		; //
		MOVWF	PCLATH				; //
		MOVF	reg_Temp1,W			; //
		ADDWF	reg0_GameLevel,W		; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F14TABBEG) - high(sa_F14TABEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
; Level figure
sa_F14TABBEG:	RETLW	020H				; Level 1 - level figure
		RETLW	060H				; Level 2 - level figure
		RETLW	070H				; Level 3 - level figure
		RETLW	0F0H				; Level 4 - level figure
		RETLW	0F8H				; Level 5 - level figure
		RETLW	0F9H				; Level 6 - level figure
		RETLW	0FDH				; Level 7 - level figure
; Game speed
		RETLW	00AH				; Level 1 - game speed
		RETLW	008H				; Level 2 - game speed
		RETLW	007H				; Level 3 - game speed
		RETLW	006H				; Level 4 - game speed
		RETLW	005H				; Level 5 - game speed
		RETLW	004H				; Level 6 - game speed
		RETLW	003H				; Level 7 - game speed
; Level step count
		RETLW	00FH				; Level 1 - step count
		RETLW	00FH				; Level 2 - step count
		RETLW	014H				; Level 3 - step count
		RETLW	014H				; Level 4 - step count
		RETLW	032H				; Level 5 - step count
		RETLW	032H				; Level 6 - step count
sa_F14TABEND:	RETLW	0FFH				; Level 7 - step count (unlimited)
;------------------------------------------------------------------------------------------------------------------------
sub_F14DISP:	BTFSC	reg0_Column0,001H		; Refresh display - the falling stars only
		BSF	reg0_LEDDriver4,004H		; // Column 0
		BTFSS	reg0_Column0,001H		; //
		BCF	reg0_LEDDriver4,004H		; //
		BTFSC	reg0_Column0,002H		; //
		BSF	reg0_LEDDriver4,005H		; //
		BTFSS	reg0_Column0,002H		; //
		BCF	reg0_LEDDriver4,005H		; //
		BTFSC	reg0_Column0,003H		; //
		BSF	reg0_LEDDriver4,006H		; //
		BTFSS	reg0_Column0,003H		; //
		BCF	reg0_LEDDriver4,006H		; //
		BTFSC	reg0_Column0,004H		; //
		BSF	reg0_LEDDriver4,007H		; //
		BTFSS	reg0_Column0,004H		; //
		BCF	reg0_LEDDriver4,007H		; //
		BTFSC	reg0_Column1,001H		; Column 1
		BSF	reg0_LEDDriver2,004H		; //
		BTFSS	reg0_Column1,001H		; //
		BCF	reg0_LEDDriver2,004H		; //
		BTFSC	reg0_Column1,002H		; //
		BSF	reg0_LEDDriver2,005H		; //
		BTFSS	reg0_Column1,002H		; //
		BCF	reg0_LEDDriver2,005H		; //
		BTFSC	reg0_Column1,003H		; //
		BSF	reg0_LEDDriver2,006H		; //
		BTFSS	reg0_Column1,003H		; //
		BCF	reg0_LEDDriver2,006H		; //
		BTFSC	reg0_Column1,004H		; //
		BSF	reg0_LEDDriver2,007H		; //
		BTFSS	reg0_Column1,004H		; //
		BCF	reg0_LEDDriver2,007H		; //
		BTFSC	reg0_Column2,001H		; Column 2
		BSF	reg0_LEDDriver4,000H		; //
		BTFSS	reg0_Column2,001H		; //
		BCF	reg0_LEDDriver4,000H		; //
		BTFSC	reg0_Column2,002H		; //
		BSF	reg0_LEDDriver4,001H		; //
		BTFSS	reg0_Column2,002H		; //
		BCF	reg0_LEDDriver4,001H		; //
		BTFSC	reg0_Column2,003H		; //
		BSF	reg0_LEDDriver4,002H		; //
		BTFSS	reg0_Column2,003H		; //
		BCF	reg0_LEDDriver4,002H		; //
		BTFSC	reg0_Column2,004H		; //
		BSF	reg0_LEDDriver4,003H		; //
		BTFSS	reg0_Column2,004H		; //
		BCF	reg0_LEDDriver4,003H		; //
		BTFSC	reg0_Column3,001H		; Column 3
		BSF	reg0_LEDDriver2,000H		; //
		BTFSS	reg0_Column3,001H		; //
		BCF	reg0_LEDDriver2,000H		; //
		BTFSC	reg0_Column3,002H		; //
		BSF	reg0_LEDDriver2,001H		; //
		BTFSS	reg0_Column3,002H		; //
		BCF	reg0_LEDDriver2,001H		; //
		BTFSC	reg0_Column3,003H		; //
		BSF	reg0_LEDDriver2,002H		; //
		BTFSS	reg0_Column3,003H		; //
		BCF	reg0_LEDDriver2,002H		; //
		BTFSC	reg0_Column3,004H		; //
		BSF	reg0_LEDDriver2,003H		; //
		BTFSS	reg0_Column3,004H		; //
		BCF	reg0_LEDDriver2,003H		; //
		RETURN					; //
;------------------------------------------------------------------------------------------------------------------------
sa_F14EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 15: Game no. 6 - basket moving
;
		BTFSS	bit_Function15			; Function requested ?
		GOTO	sa_F15EXIT			; // NO ! - skip function
		MOVLW	high(sa_F15PHASE)		; // YES ! - function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg0_F15Phase,W			; //
		ADDWF	PCL,F				; //
sa_F15PHASE:	GOTO	sa_F15PHASE0			; //
		GOTO	sa_F15PHASE1			; //
sa_F15PHEND:	GOTO	sa_F15PHASE2			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F15PHASE) - high(sa_F15PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F15PHASE0:	CALL	sub_F15DISP			; Refresh display
		INCF	reg0_F15Phase,F			; Phase 1
		GOTO	sa_F15EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F15PHASE1:	BTFSS	bit_Key				; Any key pressed ?
		GOTO	sa_F15EXIT			; // NO ! - exit function
;
		MOVLW	002H				; Key code 0x02 ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F15PH100			; // YES !
		MOVLW	004H				; Key code 0x04 ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F15PH100			; // YES !
		MOVLW	010H				; Key code 0x10 ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F15PH110			; // YES !
		MOVLW	008H				; Key code 0x08 ?
		XORWF	reg0_KeyCode,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F15PH110			; // YES !
		GOTO	sa_F15PH199			; //
;
sa_F15PH100:	BCF	STATUS,C			; Move the basket left
		RRF	reg0_G6Row,F			; //
		MOVLW	000H				; //
		XORWF	reg0_G6Row,W			; //
		MOVLW	008H				; //
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_G6Row			; //
		GOTO	sa_F15PH120			; //
;
sa_F15PH110:	BCF	STATUS,C			; Move the basket right
		RLF	reg0_G6Row,F			; //
		MOVLW	010H				; //
		XORWF	reg0_G6Row,W			; //
		MOVLW	001H				; //
		BTFSC	STATUS,Z			; //
		MOVWF	reg0_G6Row			; //
		GOTO	sa_F15PH120			; //
;
sa_F15PH120:	CALL	sub_F15DISP			; Refresh display
;
sa_F15PH199:	INCF	reg0_F15Phase,F			; Phase 2
		GOTO	sa_F15EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F15PHASE2:	BTFSS	bit_Key				; All keys released ?
		DECF	reg0_F15Phase,F			; // YES ! - phase 1
		GOTO	sa_F15EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sub_F15DISP:	BTFSC	reg0_G6Row,000H			; Refresh display - basket only (red point)
		BSF	reg0_LEDDriver3,007H		; //
		BTFSS	reg0_G6Row,000H			; //
		BCF	reg0_LEDDriver3,007H		; //
		BTFSC	reg0_G6Row,001H			; //
		BSF	reg0_LEDDriver1,007H		; //
		BTFSS	reg0_G6Row,001H			; //
		BCF	reg0_LEDDriver1,007H		; //
		BTFSC	reg0_G6Row,002H			; //
		BSF	reg0_LEDDriver3,003H		; //
		BTFSS	reg0_G6Row,002H			; //
		BCF	reg0_LEDDriver3,003H		; //
		BTFSC	reg0_G6Row,003H			; //
		BSF	reg0_LEDDriver1,003H		; //
		BTFSS	reg0_G6Row,003H			; //
		BCF	reg0_LEDDriver1,003H		; //
		RETURN					; //
;------------------------------------------------------------------------------------------------------------------------
sa_F15EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 50: Player A (computer) move calculation
;
		BTFSS	bit_Function50			; Function 50 requested ?
		GOTO	sa_F50EXIT			; // NO ! - skip function
;
		MOVLW	high(sa_F50PHASE)		; Function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg0_F50Phase,W			; //
		ADDWF	PCL,F				; //
sa_F50PHASE:	GOTO	sa_F50PHASE0			; //
		GOTO	sa_F50PHASE1			; //
		GOTO	sa_F50PHASE2			; //
		GOTO	sa_F50PHASE3			; //
		GOTO	sa_F50PHASE4			; //
		GOTO	sa_F50PHASE5			; //
sa_F50PHEND:	GOTO	sa_F50PHASE6			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F50PHASE) - high(sa_F50PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F50PHASE0:	MOVF	reg0_Random,W			; Capture pseudo-random value
		MOVWF	reg0_MoveRND			; //
		MOVLW	0FFH				; Initialize MTWA ranking register
		MOVWF	reg0_RankingA			; //
		MOVWF	reg0_Ranking			; //
		MOVWF	reg0_RankingAlt			; //
		MOVLW	00CH				; Initialize figure A attempt counter - 12 attemps in total
		MOVWF	reg0_CounterA			; //
		INCF	reg0_F50Phase,F			; Phase 1
		GOTO	sa_F50EXIT			; //
;------------------------------------------------------------------------------------------------------------------------
sa_F50PHASE1:	MOVLW	0FFH				; Initialize MTWB ranking register
		MOVWF	reg0_RankingB			; //
		MOVLW	00CH				; Initialize figure B attempt counter - 12 attemps in total
		MOVWF	reg0_CounterB			; //
		INCF	reg0_F50Phase,F			; Phase 2
		GOTO	sa_F50EXIT			; //
;------------------------------------------------------------------------------------------------------------------------
sa_F50PHASE2:	MOVF	reg0_PieceB1,W			; Transfer/restore figure B in working register due to initial
		MOVWF	reg0_PieceB1W			; // collision check
		MOVF	reg0_PieceB2,W			; //
		MOVWF	reg0_PieceB2W			; //
		MOVF	reg0_PieceB3,W			; //
		MOVWF	reg0_PieceB3W			; //
; Pseudo-random move pointer of figure A
		DECF	reg0_CounterA,W			; Figure A move attempt
		ADDWF	reg0_MoveRND,W			; // add pseudo-random value
		MOVWF	reg0_MovePtrA			; //
		MOVLW	00CH				; //
		SUBWF	reg0_MovePtrA,F			; //
		BTFSS	STATUS,C			; //
		ADDWF	reg0_MovePtrA,F			; //
		SUBWF	reg0_MovePtrA,F			; //
		BTFSS	STATUS,C			; //
		ADDWF	reg0_MovePtrA,F			; //
		MOVF	reg0_MovePtrA,W			; Piece A1 table pointer
		MOVWF	reg_Temp1			; //
		CALL	sa_F50TAB1			; Get move direction of piece A1 from table
		ADDWF	reg0_PieceA1,W			; Calculate new position of piece A1
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceA1W			; //
		MOVLW	00CH				; Piece A2 table offset
		ADDWF	reg_Temp1,F			; //
		CALL	sa_F50TAB1			; Get move direction of piece A2 from table
		ADDWF	reg0_PieceA2,W			; Calculate new position of piece A2
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceA2W			; //
		MOVLW	00CH				; Piece A3 table offset
		ADDWF	reg_Temp1,F			; //
		CALL	sa_F50TAB1			; Get move direction of piece A3 from table
		ADDWF	reg0_PieceA3,W			; Calculate new position of piece A3
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceA3W			; //
		CALL	sub_Collision			; Collision check
		MOVLW	006H				; //
		MOVWF	reg0_F50Phase			; //
		BTFSC	STATUS,Z			; Collision detected ?
		GOTO	sa_F50EXIT			; // YES ! - phase 6, skip MTW calculation
; Pseudo-random move pointer of figure B
		DECF	reg0_CounterB,W			; Figure B move attempt
		ADDWF	reg0_MoveRND,W			; // add pseudo-random value
		MOVWF	reg0_MovePtrB			; //
		MOVLW	00CH				; //
		SUBWF	reg0_MovePtrB,F			; //
		BTFSS	STATUS,C			; //
		ADDWF	reg0_MovePtrB,F			; //
		SUBWF	reg0_MovePtrB,F			; //
		BTFSS	STATUS,C			; //
		ADDWF	reg0_MovePtrB,F			; //
		MOVF	reg0_MovePtrB,W			; Piece B1 table pointer
		MOVWF	reg_Temp1			; //
		CALL	sa_F50TAB1			; Get move direction of piece B1 from table
		ADDWF	reg0_PieceB1,W			; Calculate new position of piece B1
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceB1W			; //
		MOVLW	00CH				; Piece B2 table offset
		ADDWF	reg_Temp1,F			; //
		CALL	sa_F50TAB1			; Get move direction of piece B2 from table
		ADDWF	reg0_PieceB2,W			; Calculate new position of piece B2
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceB2W			; //
		MOVLW	00CH				; Piece B3 table offset
		ADDWF	reg_Temp1,F			; //
		CALL	sa_F50TAB1			; Get move direction of piece B3 from table
		ADDWF	reg0_PieceB3,W			; Calculate new position of piece B3
		ANDLW	033H				; // mask carry
		MOVWF	reg0_PieceB3W			; //
		CALL	sub_Collision			; Collision check
		MOVLW	003H				; Collision detected ?
		BTFSC	STATUS,Z			; // YES ! - phase 5, skip MTW calculation / NO ! - phase 3
		MOVLW	005H				; // YES !
		MOVWF	reg0_F50Phase			; //
		GOTO	sa_F50EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
; Move to win calculator
sa_F50PHASE3:	MOVLW	0FFH				; Initialize temporary MTWA & MTWB register
		MOVWF	reg0_MTWA			; //
		MOVWF	reg0_MTWB			; //
		MOVLW	010H				; Initialize finish counter with value 16
		MOVWF	reg0_FinishCnt			; //
		INCF	reg0_F50Phase,F			; Phase 4
		GOTO	sa_F50EXIT			; //
;------------------------------------------------------------------------------------------------------------------------
sa_F50PHASE4:	DECF	reg0_FinishCnt,W		; Get finish 1 position from table
		MOVWF	reg_Temp1			; //
		CALL	sa_F50TAB3			; //
		MOVWF	reg0_Finish1			; //
		MOVLW	010H				; Get finish 2 position from table
		ADDWF	reg_Temp1,F			; //
		CALL	sa_F50TAB3			; //
		MOVWF	reg0_Finish2			; //
		MOVLW	010H				; Get finish 3 position from table
		ADDWF	reg_Temp1,F			; //
		CALL	sa_F50TAB3			; //
		MOVWF	reg0_Finish3			; //
;
		CALL	sub_CollFinB			; Check collision with figure B
		BTFSC	STATUS,Z			; Collision detected ?
		GOTO	sa_F50PH400			; // YES ! - skip move to win calculation
; Combination 1
		MOVF	reg0_PieceA1W,W			; Move figure A to distance register
		MOVWF	reg0_Figure1			; //
		MOVF	reg0_PieceA2W,W			; //
		MOVWF	reg0_Figure2			; //
		MOVF	reg0_PieceA3W,W			; //
		MOVWF	reg0_Figure3			; //
		CALL	sub_F50DIST			; Calculate MTW
		SUBWF	reg0_MTWA,W			; Compare calculated MTW with last known value
		MOVF	reg_Temp1,W			; //
		BTFSC	STATUS,C			; //
		MOVWF	reg0_MTWA			; Update lowest MTW value
; Combination 2
		MOVF	reg0_PieceA1W,W			; Move figure A to distance register
		MOVWF	reg0_Figure1			; //
		MOVF	reg0_PieceA2W,W			; //
		MOVWF	reg0_Figure3			; //
		MOVF	reg0_PieceA3W,W			; //
		MOVWF	reg0_Figure2			; //
		CALL	sub_F50DIST			; Calculate MTW
		SUBWF	reg0_MTWA,W			; Compare calculated MTW with last known value
		MOVF	reg_Temp1,W			; //
		BTFSC	STATUS,C			; //
		MOVWF	reg0_MTWA			; Update lowest MTW value
; Combination 3
		MOVF	reg0_PieceA1W,W			; Move figure A to distance register
		MOVWF	reg0_Figure2			; //
		MOVF	reg0_PieceA2W,W			; //
		MOVWF	reg0_Figure1			; //
		MOVF	reg0_PieceA3W,W			; //
		MOVWF	reg0_Figure3			; //
		CALL	sub_F50DIST			; Calculate MTW
		SUBWF	reg0_MTWA,W			; Compare calculated MTW with last known value
		MOVF	reg_Temp1,W			; //
		BTFSC	STATUS,C			; //
		MOVWF	reg0_MTWA			; Update lowest MTW value
; Combination 4
		MOVF	reg0_PieceA1W,W			; Move figure A to distance register
		MOVWF	reg0_Figure2			; //
		MOVF	reg0_PieceA2W,W			; //
		MOVWF	reg0_Figure3			; //
		MOVF	reg0_PieceA3W,W			; //
		MOVWF	reg0_Figure1			; //
		CALL	sub_F50DIST			; Calculate MTW
		SUBWF	reg0_MTWA,W			; Compare calculated MTW with last known value
		MOVF	reg_Temp1,W			; //
		BTFSC	STATUS,C			; //
		MOVWF	reg0_MTWA			; Update lowest MTW value
; Combination 5
		MOVF	reg0_PieceA1W,W			; Move figure A to distance register
		MOVWF	reg0_Figure3			; //
		MOVF	reg0_PieceA2W,W			; //
		MOVWF	reg0_Figure1			; //
		MOVF	reg0_PieceA3W,W			; //
		MOVWF	reg0_Figure2			; //
		CALL	sub_F50DIST			; Calculate MTW
		SUBWF	reg0_MTWA,W			; Compare calculated MTW with last known value
		MOVF	reg_Temp1,W			; //
		BTFSC	STATUS,C			; //
		MOVWF	reg0_MTWA			; Update lowest MTW value
; Combination 6
		MOVF	reg0_PieceA1W,W			; Move figure A to distance register
		MOVWF	reg0_Figure3			; //
		MOVF	reg0_PieceA2W,W			; //
		MOVWF	reg0_Figure2			; //
		MOVF	reg0_PieceA3W,W			; //
		MOVWF	reg0_Figure1			; //
		CALL	sub_F50DIST			; Calculate MTW
		SUBWF	reg0_MTWA,W			; Compare calculated MTW with last known value
		MOVF	reg_Temp1,W			; //
		BTFSC	STATUS,C			; //
		MOVWF	reg0_MTWA			; Update lowest MTW value
;
sa_F50PH400:	CALL	sub_CollFinA			; Check collision with figure A
		BTFSC	STATUS,Z			; Collision detected ?
		GOTO	sa_F50PH410			; // YES ! - skip move to win calculation
; Combination 1
		MOVF	reg0_PieceB1W,W			; Move figure B to distance register
		MOVWF	reg0_Figure1			; //
		MOVF	reg0_PieceB2W,W			; //
		MOVWF	reg0_Figure2			; //
		MOVF	reg0_PieceB3W,W			; //
		MOVWF	reg0_Figure3			; //
		CALL	sub_F50DIST			; Calculate MTW
		CALL	sub_F50CORR			; MTWB correction
		SUBWF	reg0_MTWB,W			; Compare calculated MTW with last known value
		MOVF	reg_Temp1,W			; //
		BTFSC	STATUS,C			; //
		MOVWF	reg0_MTWB			; Update lowest MTW value
; Combination 2
		MOVF	reg0_PieceB1W,W			; Move figure B to distance register
		MOVWF	reg0_Figure1			; //
		MOVF	reg0_PieceB2W,W			; //
		MOVWF	reg0_Figure3			; //
		MOVF	reg0_PieceB3W,W			; //
		MOVWF	reg0_Figure2			; //
		CALL	sub_F50DIST			; Calculate MTW
		CALL	sub_F50CORR			; MTWB correction
		SUBWF	reg0_MTWB,W			; Compare calculated MTW with last known value
		MOVF	reg_Temp1,W			; //
		BTFSC	STATUS,C			; //
		MOVWF	reg0_MTWB			; Update lowest MTW value
; Combination 3
		MOVF	reg0_PieceB1W,W			; Move figure B to distance register
		MOVWF	reg0_Figure2			; //
		MOVF	reg0_PieceB2W,W			; //
		MOVWF	reg0_Figure1			; //
		MOVF	reg0_PieceB3W,W			; //
		MOVWF	reg0_Figure3			; //
		CALL	sub_F50DIST			; Calculate MTW
		CALL	sub_F50CORR			; MTWB correction
		SUBWF	reg0_MTWB,W			; Compare calculated MTW with last known value
		MOVF	reg_Temp1,W			; //
		BTFSC	STATUS,C			; //
		MOVWF	reg0_MTWB			; Update lowest MTW value
; Combination 4
		MOVF	reg0_PieceB1W,W			; Move figure B to distance register
		MOVWF	reg0_Figure2			; //
		MOVF	reg0_PieceB2W,W			; //
		MOVWF	reg0_Figure3			; //
		MOVF	reg0_PieceB3W,W			; //
		MOVWF	reg0_Figure1			; //
		CALL	sub_F50DIST			; Calculate MTW
		CALL	sub_F50CORR			; MTWB correction
		SUBWF	reg0_MTWB,W			; Compare calculated MTW with last known value
		MOVF	reg_Temp1,W			; //
		BTFSC	STATUS,C			; //
		MOVWF	reg0_MTWB			; Update lowest MTW value
; Combination 5
		MOVF	reg0_PieceB1W,W			; Move figure B to distance register
		MOVWF	reg0_Figure3			; //
		MOVF	reg0_PieceB2W,W			; //
		MOVWF	reg0_Figure1			; //
		MOVF	reg0_PieceB3W,W			; //
		MOVWF	reg0_Figure2			; //
		CALL	sub_F50DIST			; Calculate MTW
		CALL	sub_F50CORR			; MTWB correction
		SUBWF	reg0_MTWB,W			; Compare calculated MTW with last known value
		MOVF	reg_Temp1,W			; //
		BTFSC	STATUS,C			; //
		MOVWF	reg0_MTWB			; Update lowest MTW value
; Combination 6
		MOVF	reg0_PieceB1W,W			; Move figure B to distance register
		MOVWF	reg0_Figure3			; //
		MOVF	reg0_PieceB2W,W			; //
		MOVWF	reg0_Figure2			; //
		MOVF	reg0_PieceB3W,W			; //
		MOVWF	reg0_Figure1			; //
		CALL	sub_F50DIST			; Calculate MTW
		CALL	sub_F50CORR			; MTWB correction
		SUBWF	reg0_MTWB,W			; Compare calculated MTW with last known value
		MOVF	reg_Temp1,W			; //
		BTFSC	STATUS,C			; //
		MOVWF	reg0_MTWB			; Update lowest MTW value
;
sa_F50PH410:	DECFSZ	reg0_FinishCnt,F		; All finish figures checked ?
		GOTO	sa_F50EXIT			; // NO ! - continue checking, phase 4
; Choose the best configuration for player B
		RLF	reg0_MTWB,W			; Get new MTWB ranking
		MOVWF	reg_Temp1			; // calculate 6 * MTWB + MTWA
		RLF	reg_Temp1,W			; //
		ANDLW	0FCH				; // 4 * MTWB
		ADDWF	reg0_MTWB,W			; //
		ADDWF	reg0_MTWB,W			; // 6 * MTWB
		ADDWF	reg0_MTWA,W			; // + MTWA
		MOVWF	reg_Temp1			; //
		CALL	sa_F50RANK			; // MTWB & MTWA ranking -> Wreg
		MOVWF	reg_Temp1			; Split MTWB & MTWA
		MOVWF	reg_Temp2			; //
		SWAPF	reg_Temp2,F			; //
		MOVLW	00FH				; //
		ANDWF	reg_Temp1,F			; //
		ANDWF	reg_Temp2,F			; //
;
		MOVF	reg_Temp2,W			; New MTWB ranking bettter than the old one ?
		SUBWF	reg0_RankingB,W			; //
		BTFSS	STATUS,C			; //
		GOTO	sa_F50PH420			; // NO !
		MOVF	reg_Temp2,W			; // YES ! - update MTWB ranking
		MOVWF	reg0_RankingB			; //
		MOVF	reg_Temp1,W			; // update MTWB ranking
		MOVWF	reg0_RankingA			; //
;
sa_F50PH420:	INCF	reg0_F50Phase,F			; Phase 5
		GOTO	sa_F50EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F50PHASE5:	MOVLW	002H				; All figure B move attempts checked ?
		MOVWF	reg0_F50Phase			; //
		DECFSZ	reg0_CounterB,F			; //
		GOTO	sa_F50EXIT			; // NO ! - phase 2, next figure B move attempt
; Choose the best configuration for player A
		MOVF	reg0_RankingA,W			; New MTWA ranking bettter or equal than the old one ?
		SUBWF	reg0_Ranking,W			; //
		BTFSS	STATUS,C			; //
		GOTO	sa_F50PH500			; // NO !
;
		MOVF	reg0_RankingA,W			; // YES ! - update MTWA ranking
		MOVWF	reg0_Ranking			; //
		MOVF	reg0_MoveCode,W			; Update alternative MTWA ranking
		MOVWF	reg0_MoveCodeAlt		; //
		INCF	reg0_MovePtrA,W			; Capture move code of player A
		MOVWF	reg0_MoveCode			; // valid move code range 0x01 - 0x12
		GOTO	sa_F50PH510			; //
;
sa_F50PH500:	MOVF	reg0_RankingA,W			; New alt. MTWA ranking bettter or equal than the old one ?
		SUBWF	reg0_RankingAlt,W		; //
		BTFSS	STATUS,C			; //
		GOTO	sa_F50PH510			; // NO !
;
		MOVF	reg0_RankingA,W			; // YES ! - update alternative MTWA ranking
		MOVWF	reg0_RankingAlt			; //
		INCF	reg0_MovePtrA,W			; Capture alternative move code of player A
		MOVWF	reg0_MoveCodeAlt		; // valid move code range 0x01 - 0x12
		GOTO	sa_F50PH510			; //
;
sa_F50PH510:	MOVLW	006H				; Phase 6
		MOVWF	reg0_F50Phase			; //
		DECFSZ	reg0_CounterB,F			; //
		GOTO	sa_F50EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F50PHASE6:	MOVLW	001H				; All figure A move attempts checked ?
		MOVWF	reg0_F50Phase			; //
		DECFSZ	reg0_CounterA,F			; //
		GOTO	sa_F50EXIT			; // NO ! - phase 1, next figure B move attempt
;
		MOVF	reg0_MCBuff3,W			; Loop detection and avoiding @ player A (computer)
		MOVWF	reg0_MCBuff4			; // update 4-stage move code buffer
		MOVF	reg0_MCBuff2,W			; //
		MOVWF	reg0_MCBuff3			; //
		MOVF	reg0_MCBuff1,W			; //
		MOVWF	reg0_MCBuff2			; //
		MOVF	reg0_MoveCode,W			; //
		MOVWF	reg0_MCBuff1			; //
;
		MOVLW	005H				; Ranking A > 0x04
		SUBWF	reg0_Ranking,W			; //
		BTFSC	STATUS,C			; //
		GOTO	sa_F50PH610			; // YES ! - skip loop detection and avoiding
;
		MOVF	reg0_MCBuff1,W			; // Buff1 = Buff3 ?
		SUBWF	reg0_MCBuff3,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F50PH610			; // NO ! - no loop
;
		RRF	reg0_MCBuff1,W			; Move code value odd or even ?
		BTFSS	STATUS,C			; //
		GOTO	sa_F50PH600			; //
		MOVF	reg0_MCBuff1,W			; Odd -> check if Buff2 - Buff1 = 0x01
		SUBWF	reg0_MCBuff2,W			; //
		XORLW	001H				; //
		BTFSS	STATUS,Z			; Loop detected ?
		GOTO	sa_F50PH610			; // NO !
		GOTO	sa_F50PH605			; // YES !
sa_F50PH600:	MOVF	reg0_MCBuff2,W			; Even -> check if Buff1 - Buff2 = 0x01
		SUBWF	reg0_MCBuff1,W			; //
		XORLW	001H				; //
		BTFSS	STATUS,Z			; Loop detected ?
		GOTO	sa_F50PH610			; // NO !
;
sa_F50PH605:	MOVF	reg0_MoveCodeAlt,W		; Loop detected -> apply alternative move code to avoid further
		MOVWF	reg0_MoveCode			; // looping
;
sa_F50PH610:	BCF	bit_Function50			; Stop function
		GOTO	sa_F50EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F50TAB1:	MOVLW	high(sa_F50TAB1BEG)		; Move direction decoder
		MOVWF	PCLATH				; //
		MOVF	reg_Temp1,W			; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F50TAB1BEG) - high(sa_F50TAB1END) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F50TAB1BEG:	RETLW	000H				; Move piece A1/B1
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	030H				; // -1 step Y-axis
		RETLW	010H				; // +1 step Y-axis
		RETLW	003H				; // -1 step X-axis
		RETLW	001H				; // +1 step X-axis
		RETLW	000H				; Move piece A2/B2
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	030H				; // -1 step Y-axis
		RETLW	010H				; // +1 step Y-axis
		RETLW	003H				; // -1 step X-axis
		RETLW	001H				; // +1 step X-axis
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	030H				; Move piece A3/B3;  -1 step Y-axis
		RETLW	010H				; // +1 step Y-axis
		RETLW	003H				; // -1 step X-axis
		RETLW	001H				; // +1 step X-axis
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	000H				; //
		RETLW	000H				; //
sa_F50TAB1END:	RETLW	000H				; //
;------------------------------------------------------------------------------------------------------------------------
sa_F50RANK:	MOVLW	high(sa_F50TAB2BEG)		; MTWB & MTWA ranking table
		MOVWF	PCLATH				; //
		MOVF	reg_Temp1,W			; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F50TAB2BEG) - high(sa_F50TAB2END) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
;							  |   MTWB  | MTWA | RankingB | RankingA |
;							  ----------------------------------------
sa_F50TAB2BEG:	RETLW	0F0H				; | 00 *(00)|  00  |    15    |     0    |
		RETLW	00FH				; | 00 *(00)|  01  |     0    |    15    |
		RETLW	00FH				; | 00 *(00)|  02  |     0    |    15    |
		RETLW	00FH				; | 00 *(00)|  03  |     0    |    15    |
		RETLW	00FH				; | 00 *(00)|  04  |     0    |    15    |
		RETLW	00FH				; | 00 *(00)|  05  |     0    |    15    |
;							  ----------------------------------------
		RETLW	0F0H				; | 01 *(01)|  00  |    15    |     0    |
		RETLW	0E1H				; | 01 *(01)|  01  |    14    |     1    |
		RETLW	01EH				; | 01 *(01)|  02  |     1    |    14    |
		RETLW	01EH				; | 01 *(01)|  03  |     1    |    14    |
		RETLW	01EH				; | 01 *(01)|  04  |     1    |    14    |
		RETLW	01EH				; | 01 *(01)|  05  |     1    |    14    |
;							  ----------------------------------------
		RETLW	0F0H				; | 02 *(01)|  00  |    15    |     0    |
		RETLW	0E1H				; | 02 *(01)|  01  |    14    |     1    |
		RETLW	04BH				; | 02 *(01)|  02  |     4    |    11    |
		RETLW	03CH				; | 02 *(01)|  03  |     3    |    12    |
		RETLW	02DH				; | 02 *(01)|  04  |     2    |    13    |
		RETLW	02DH				; | 02 *(01)|  05  |     2    |    13    |
;							  ----------------------------------------
		RETLW	0F0H				; | 03 *(02)|  00  |    15    |     0    |
		RETLW	0E1H				; | 03 *(02)|  01  |    14    |     1    |
		RETLW	074H				; | 03 *(02)|  02  |     7    |     4    |
		RETLW	067H				; | 03 *(02)|  03  |     6    |     7    |
		RETLW	05AH				; | 03 *(02)|  04  |     5    |    10    |
		RETLW	05AH				; | 03 *(02)|  05  |     5    |    10    |
;							  ----------------------------------------
		RETLW	0F0H				; | 04 *(03)|  00  |    15    |     0    |
		RETLW	0E1H				; | 04 *(03)|  01  |    14    |     1    |
		RETLW	0A3H				; | 04 *(03)|  02  |    10    |     3    |
		RETLW	096H				; | 04 *(03)|  03  |     9    |     6    |
		RETLW	089H				; | 04 *(03)|  04  |     8    |     9    |
		RETLW	089H				; | 04 *(03)|  05  |     8    |     9    |
;							  ----------------------------------------
		RETLW	0F0H				; | 05 *(04)|  00  |    15    |     0    |
		RETLW	0E1H				; | 05 *(04)|  01  |    14    |     1    |
		RETLW	0D2H				; | 05 *(04)|  02  |    13    |     2    |
		RETLW	0C5H				; | 05 *(04)|  03  |    12    |     5    |
		RETLW	0B8H				; | 05 *(04)|  04  |    11    |     8    |
		RETLW	0B8H				; | 05 *(04)|  05  |    11    |     8    |
;							  ----------------------------------------
		RETLW	0F0H				; | 06 *(05)|  00  |    15    |     0    |
		RETLW	0E1H				; | 06 *(05)|  01  |    14    |     1    |
		RETLW	0D2H				; | 06 *(05)|  02  |    13    |     2    |
		RETLW	0C5H				; | 06 *(05)|  03  |    12    |     5    |
		RETLW	0B8H				; | 06 *(05)|  04  |    11    |     8    |
sa_F50TAB2END:	RETLW	0B8H				; | 06 *(05)|  05  |    11    |     8    | *) real MTWB value
;------------------------------------------------------------------------------------------------------------------------
; Collision check figure A & figure B
; collision detected -> Z = 1 / collision not detected -> Z = 0
sub_Collision:	MOVF	reg0_PieceA3W,W			; Collision check
		XORWF	reg0_PieceA2W,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA3W,W			; //
		XORWF	reg0_PieceA1W,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA2W,W			; //
		XORWF	reg0_PieceA1W,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
;
		MOVF	reg0_PieceA3W,W			; Collision check
		XORWF	reg0_PieceB3W,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA3W,W			; //
		XORWF	reg0_PieceB2W,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA3W,W			; //
		XORWF	reg0_PieceB1W,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
;
		MOVF	reg0_PieceA2W,W			; Collision check
		XORWF	reg0_PieceB3W,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA2W,W			; //
		XORWF	reg0_PieceB2W,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA2W,W			; //
		XORWF	reg0_PieceB1W,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
;
		MOVF	reg0_PieceA1W,W			; Collision check
		XORWF	reg0_PieceB3W,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA1W,W			; //
		XORWF	reg0_PieceB2W,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA1W,W			; //
		XORWF	reg0_PieceB1W,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
;
		MOVF	reg0_PieceB3W,W			; Collision check
		XORWF	reg0_PieceB2W,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceB3W,W			; //
		XORWF	reg0_PieceB1W,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceB2W,W			; //
		XORWF	reg0_PieceB1W,W			; //
		RETURN					; //
;------------------------------------------------------------------------------------------------------------------------
sa_F50TAB3:	MOVLW	high(sa_F50TAB3BEG)		; Finish figure decoder
		MOVWF	PCLATH				; //
		MOVF	reg_Temp1,W			; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F50TAB3BEG) - high(sa_F50TAB3END) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F50TAB3BEG:	RETLW	000H				; Finish 1 position
		RETLW	001H				; //
		RETLW	002H				; //
		RETLW	003H				; //
		RETLW	010H				; //
		RETLW	011H				; //
		RETLW	012H				; //
		RETLW	013H				; //
		RETLW	000H				; //
		RETLW	010H				; //
		RETLW	020H				; //
		RETLW	030H				; //
		RETLW	001H				; //
		RETLW	011H				; //
		RETLW	021H				; //
		RETLW	031H				; //
		RETLW	010H				; Finish 2 position
		RETLW	011H				; //
		RETLW	012H				; //
		RETLW	013H				; //
		RETLW	020H				; //
		RETLW	021H				; //
		RETLW	022H				; //
		RETLW	023H				; //
		RETLW	001H				; //
		RETLW	011H				; //
		RETLW	021H				; //
		RETLW	031H				; //
		RETLW	002H				; //
		RETLW	012H				; //
		RETLW	022H				; //
		RETLW	032H				; //
		RETLW	020H				; Finish 3 position
		RETLW	021H				; //
		RETLW	022H				; //
		RETLW	023H				; //
		RETLW	030H				; //
		RETLW	031H				; //
		RETLW	032H				; //
		RETLW	033H				; //
		RETLW	002H				; //
		RETLW	012H				; //
		RETLW	022H				; //
		RETLW	032H				; //
		RETLW	003H				; //
		RETLW	013H				; //
		RETLW	023H				; //
sa_F50TAB3END:	RETLW	033H				; //
;------------------------------------------------------------------------------------------------------------------------
sub_F50DIST:	CLRF	reg_Temp1			; Clear move to win accumulator
		MOVF	reg0_Figure1,W			; Subtract high nibbles
		ANDLW	0F0H				; //
		SUBWF	reg0_Finish1,W			; //
		BTFSS	STATUS,C			; High nibble two's complement
		XORLW	0FFH				; //
		BTFSS	STATUS,C			; //
		ADDLW	010H				; //
		ANDLW	0F0H				; //
		ADDWF	reg_Temp1,F			; //
		XORLW	030H				; If result equal 0x10 or 0x30 -> distance equal 0x10
		MOVLW	020H				; //
		BTFSC	STATUS,Z			; //
		SUBWF	reg_Temp1,F			; //
		MOVF	reg0_Figure1,W			; Subtract low nibbles
		SUBWF	reg0_Finish1,W			; //
		BTFSS	STATUS,DC			; Low nibble two's complement
		XORLW	0FFH				; //
		BTFSS	STATUS,DC			; //
		ADDLW	001H				; //
		ANDLW	00FH				; //
		ADDWF	reg_Temp1,F			; //
		XORLW	003H				; If result equal 0x01 or 0x03 -> distance equal 0x01
		MOVLW	002H				; //
		BTFSC	STATUS,Z			; //
		SUBWF	reg_Temp1,F			; //
;
		MOVF	reg0_Figure2,W			; Subtract high nibbles
		ANDLW	0F0H				; //
		SUBWF	reg0_Finish2,W			; //
		BTFSS	STATUS,C			; High nibble two's complement
		XORLW	0FFH				; //
		BTFSS	STATUS,C			; //
		ADDLW	010H				; //
		ANDLW	0F0H				; //
		ADDWF	reg_Temp1,F			; //
		XORLW	030H				; If result equal 0x10 or 0x30 -> distance equal 0x10
		MOVLW	020H				; //
		BTFSC	STATUS,Z			; //
		SUBWF	reg_Temp1,F			; //
		MOVF	reg0_Figure2,W			; Subtract low nibbles
		SUBWF	reg0_Finish2,W			; //
		BTFSS	STATUS,DC			; Low nibble two's complement
		XORLW	0FFH				; //
		BTFSS	STATUS,DC			; //
		ADDLW	001H				; //
		ANDLW	00FH				; //
		ADDWF	reg_Temp1,F			; //
		XORLW	003H				; If result equal 0x01 or 0x03 -> distance equal 0x01
		MOVLW	002H				; //
		BTFSC	STATUS,Z			; //
		SUBWF	reg_Temp1,F			; //
;
		MOVF	reg0_Figure3,W			; Subtract high nibbles
		ANDLW	0F0H				; //
		SUBWF	reg0_Finish3,W			; //
		BTFSS	STATUS,C			; High nibble two's complement
		XORLW	0FFH				; //
		BTFSS	STATUS,C			; //
		ADDLW	010H				; //
		ANDLW	0F0H				; //
		ADDWF	reg_Temp1,F			; //
		XORLW	030H				; If result equal 0x10 or 0x30 -> distance equal 0x10
		MOVLW	020H				; //
		BTFSC	STATUS,Z			; //
		SUBWF	reg_Temp1,F			; //
		MOVF	reg0_Figure3,W			; Subtract low nibbles
		SUBWF	reg0_Finish3,W			; //
		BTFSS	STATUS,DC			; Low nibble two's complement
		XORLW	0FFH				; //
		BTFSS	STATUS,DC			; //
		ADDLW	001H				; //
		ANDLW	00FH				; //
		ADDWF	reg_Temp1,F			; //
		XORLW	003H				; If result equal 0x01 or 0x03 -> distance equal 0x01
		MOVLW	002H				; //
		BTFSC	STATUS,Z			; //
		SUBWF	reg_Temp1,F			; //
;
		SWAPF	reg_Temp1,W			; Calculate final MTW
		ADDWF	reg_Temp1,W			; //
		ANDLW	00FH				; //
		MOVWF	reg_Temp1			; //
		RETURN					; //
;------------------------------------------------------------------------------------------------------------------------
sub_F50CORR:	MOVLW	000H				; MTWB = 0 ?
		XORWF	reg_Temp1,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F50CORR10			; // YES ! - no correction
;
		BTFSS	bit_Difficulty			; Game difficulty low ?
		GOTO	sa_F50CORR00			; // YES ! - skip conditional correction
;
		MOVLW	001H				; MTWB = 1 ?
		XORWF	reg_Temp1,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F50CORR20			; // YES ! - conditional correction
;
sa_F50CORR00:	INCF	reg_Temp1,W			; MTWB > 1 or difficulty low -> always correction
		MOVWF	reg_Temp1			; //
		RETURN					; //
;
sa_F50CORR10:	MOVF	reg_Temp1,W			; No correction
		MOVWF	reg_Temp1			; //
		RETURN					; //
;
sa_F50CORR20:	MOVF	reg0_Finish1,W			; Find finish figure field to be checked
		MOVWF	reg_Temp2			; //
		XORWF	reg0_Figure1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F50CORR30			; //
		MOVF	reg0_Finish2,W			; //
		MOVWF	reg_Temp2			; //
		XORWF	reg0_Figure2,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F50CORR30			; //
		MOVF	reg0_Finish3,W			; //
		MOVWF	reg_Temp2			; //
;
sa_F50CORR30:	MOVF	reg_Temp2,W			; Subtract high nibbles
		ANDLW	0F0H				; //
		SUBWF	reg0_PieceA1W,W			; //
		BTFSS	STATUS,C			; High nibble two's complement
		XORLW	0FFH				; //
		BTFSS	STATUS,C			; //
		ADDLW	010H				; //
		ANDLW	0F0H				; //
		MOVWF	reg_Temp1			; //
		XORLW	030H				; If result equal 0x10 or 0x30 -> distance equal 0x10
		MOVLW	020H				; //
		BTFSC	STATUS,Z			; //
		SUBWF	reg_Temp1,F			; //
		MOVF	reg_Temp2,W			; Subtract low nibbles
		SUBWF	reg0_PieceA1W,W			; //
		BTFSS	STATUS,DC			; Low nibble two's complement
		XORLW	0FFH				; //
		BTFSS	STATUS,DC			; //
		ADDLW	001H				; //
		ANDLW	00FH				; //
		ADDWF	reg_Temp1,F			; //
		XORLW	003H				; If result equal 0x01 or 0x03 -> distance equal 0x01
		MOVLW	002H				; //
		BTFSC	STATUS,Z			; //
		SUBWF	reg_Temp1,F			; //
		SWAPF	reg_Temp1,W			; Calculate distance
		ADDWF	reg_Temp1,W			; //
		ANDLW	00FH				; //
		MOVWF	reg_Temp1			; //
		XORLW	001H				; Distance to piece B1 = 1 ?
		BTFSC	STATUS,Z			; //
		GOTO	sa_F50CORR40			; // YES !
;
		MOVF	reg_Temp2,W			; Subtract high nibbles
		ANDLW	0F0H				; //
		SUBWF	reg0_PieceA2W,W			; //
		BTFSS	STATUS,C			; High nibble two's complement
		XORLW	0FFH				; //
		BTFSS	STATUS,C			; //
		ADDLW	010H				; //
		ANDLW	0F0H				; //
		MOVWF	reg_Temp1			; //
		XORLW	030H				; If result equal 0x10 or 0x30 -> distance equal 0x10
		MOVLW	020H				; //
		BTFSC	STATUS,Z			; //
		SUBWF	reg_Temp1,F			; //
		MOVF	reg_Temp2,W			; Subtract low nibbles
		SUBWF	reg0_PieceA2W,W			; //
		BTFSS	STATUS,DC			; Low nibble two's complement
		XORLW	0FFH				; //
		BTFSS	STATUS,DC			; //
		ADDLW	001H				; //
		ANDLW	00FH				; //
		ADDWF	reg_Temp1,F			; //
		XORLW	003H				; If result equal 0x01 or 0x03 -> distance equal 0x01
		MOVLW	002H				; //
		BTFSC	STATUS,Z			; //
		SUBWF	reg_Temp1,F			; //
		SWAPF	reg_Temp1,W			; Calculate distance
		ADDWF	reg_Temp1,W			; //
		ANDLW	00FH				; //
		MOVWF	reg_Temp1			; //
		XORLW	001H				; Distance to piece B2 = 1 ?
		BTFSC	STATUS,Z			; //
		GOTO	sa_F50CORR40			; // YES !
;
		MOVF	reg_Temp2,W			; Subtract high nibbles
		ANDLW	0F0H				; //
		SUBWF	reg0_PieceA3W,W			; //
		BTFSS	STATUS,C			; High nibble two's complement
		XORLW	0FFH				; //
		BTFSS	STATUS,C			; //
		ADDLW	010H				; //
		ANDLW	0F0H				; //
		MOVWF	reg_Temp1			; //
		XORLW	030H				; If result equal 0x10 or 0x30 -> distance equal 0x10
		MOVLW	020H				; //
		BTFSC	STATUS,Z			; //
		SUBWF	reg_Temp1,F			; //
		MOVF	reg_Temp2,W			; Subtract low nibbles
		SUBWF	reg0_PieceA3W,W			; //
		BTFSS	STATUS,DC			; Low nibble two's complement
		XORLW	0FFH				; //
		BTFSS	STATUS,DC			; //
		ADDLW	001H				; //
		ANDLW	00FH				; //
		ADDWF	reg_Temp1,F			; //
		XORLW	003H				; If result equal 0x01 or 0x03 -> distance equal 0x01
		MOVLW	002H				; //
		BTFSC	STATUS,Z			; //
		SUBWF	reg_Temp1,F			; //
		SWAPF	reg_Temp1,W			; Calculate distance
		ADDWF	reg_Temp1,W			; //
		ANDLW	00FH				; //
		MOVWF	reg_Temp1			; //
		XORLW	001H				; Distance to piece B3 = 1 ?
		BTFSC	STATUS,Z			; //
		GOTO	sa_F50CORR40			; // YES !
		MOVLW	001H				; // NO ! - distance Bx > 1 -> no MTWB correction (MTWB = 1)
		MOVWF	reg_Temp1			; //
		RETURN					; //
;				
sa_F50CORR40:	MOVLW	002H				; MTWB correction (MTWB = 2)
		MOVWF	reg_Temp1			; //
		RETURN					; //
;------------------------------------------------------------------------------------------------------------------------
; Collision check figure A & finish A
; collision detected -> Z = 1 / collision not detected -> Z = 0
sub_CollFinA:	MOVF	reg0_PieceA1W,W			; Collision check
		XORWF	reg0_Finish1,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA1W,W			; //
		XORWF	reg0_Finish2,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA1W,W			; //
		XORWF	reg0_Finish3,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA2W,W			; Collision check
		XORWF	reg0_Finish1,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA2W,W			; //
		XORWF	reg0_Finish2,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA2W,W			; //
		XORWF	reg0_Finish3,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA3W,W			; Collision check
		XORWF	reg0_Finish1,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA3W,W			; //
		XORWF	reg0_Finish2,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceA3W,W			; //
		XORWF	reg0_Finish3,W			; //
		RETURN					; //
;------------------------------------------------------------------------------------------------------------------------
; Collision check figure B & finish B
; collision detected -> Z = 1 / collision not detected -> Z = 0
sub_CollFinB:	MOVF	reg0_PieceB1W,W			; Collision check
		XORWF	reg0_Finish1,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceB1W,W			; //
		XORWF	reg0_Finish2,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceB1W,W			; //
		XORWF	reg0_Finish3,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceB2W,W			; Collision check
		XORWF	reg0_Finish1,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceB2W,W			; //
		XORWF	reg0_Finish2,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceB2W,W			; //
		XORWF	reg0_Finish3,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceB3W,W			; Collision check
		XORWF	reg0_Finish1,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceB3W,W			; //
		XORWF	reg0_Finish2,W			; //
		BTFSC	STATUS,Z			; Collision detected ?
		RETURN					; // YES ! - Z = 1
		MOVF	reg0_PieceB3W,W			; //
		XORWF	reg0_Finish3,W			; //
		RETURN					; //
;------------------------------------------------------------------------------------------------------------------------
sa_F50EXIT						; Function delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;
;
		mPRGMEMPAGE0				; --> PROGRAM MEMORY PAGE 0
		GOTO	sa_MAINLOOP			; Back to mainloop begin
;
;************************************************************************************************************************
;*		A P P L I C A T I O N     E N D										*
;************************************************************************************************************************
;
;
;
		END
;
; GRID game playground
; Player A (user or computer) - pieces A1 / A2/ A3
; Player B (user only) - pieces B1 / B2 / B3
;
;	      Y-axis
;		|
;		|
;
;		-------------------------------------------------------------------
;		|-----------------------------------------------------------------|
;		||               |               |               |               ||
;		||               |               |               |               ||
;	3x	||               |               |      *A2*     |      *B1*     ||
;		||               |               |               |               ||
;		||            30 |            31 |            32 |            33 ||
;		|-----------------------------------------------------------------|
;		||               |               |               |               ||
;		||               |               |               |               ||
;	2x	||               |               |               |      *A3*     ||
;		||               |               |               |               ||
;		||            20 |            21 |            22 |            23 ||
;		|-----------------------------------------------------------------|
;		||               |               |               |               ||
;		||               |               |               |               ||
;	1x	||      *B3*     |               |               |               ||
;		||               |               |               |               ||
;		||            10 |            11 |            12 |            13 ||
;		|-----------------------------------------------------------------|
;		||               |               |               |               ||
;		||               |               |               |               ||
;	0x	||      *A1*     |      *B2*     |               |               ||
;		||               |               |               |               ||
;		||            00 |            01 |            02 |            03 ||
;		|-----------------------------------------------------------------|
;		------------------------------------------------------------------- -- X-axis
;
;	YX               y0              y1              y2              y3
;
;
;
;
; Component layout
;
;	-------------------------------------------------
;	|  S101                                   S100  |
;	| (0x04)                                 (0x08) |
;	|                                               |
;	|         LD212   LD204   LD208   LD200         |
;	|                                               |
;	|                                               |
;	|         LD213   LD205   LD209   LD201         |
;	|                                               |
;	|                                               |
;	|         LD214   LD206   LD210   LD202         |
;	|                                               |
;	|                                               |
;	|         LD215   LD207   LD211   LD203         |
;	|                                               |
;	| (0x02)                                 (0x10) |
;	|  S103                                   S102  |
;	|                                               |
;	|                                               |
;	|                                               |
;
;
;	      COLx        |  COL8  |  COL7  |  COL6  |  COL5  |  COL4  |  COL3  |  COL2  |  COL1  |
;	-------------------------------------------------------------------------------------------
;	      bit         |    7.  |    6.  |    5.  |    4.  |    3.  |    2.  |    1.  |    0.  |
;	-------------------------------------------------------------------------------------------
; ROW2	reg0_LEDDriver1   | LD207R | LD206R | LD205R | LD204R | LD203R | LD202R | LD201R | LD200R |
;	-------------------------------------------------------------------------------------------
; ROW3	reg0_LEDDriver2   | LD207Y | LD206Y | LD205Y | LD204Y | LD203Y | LD202Y | LD201Y | LD200Y |
;	-------------------------------------------------------------------------------------------
; ROW3	reg0_LEDDriver3   | LD215R | LD214R | LD213R | LD212R | LD211R | LD210R | LD209R | LD208R |
;	-------------------------------------------------------------------------------------------
; ROW4	reg0_LEDDriver4   | LD215Y | LD214Y | LD213Y | LD212Y | LD211Y | LD210Y | LD209Y | LD208Y |
;	-------------------------------------------------------------------------------------------
;
